###########################################################################
# 
#  Psyco general support module.
#   Copyright (C) 2001-2002  Armin Rigo et.al.

"""Psyco general support module.

For internal use.
"""
###########################################################################

import sys, _psyco, __builtin__

error = _psyco.error
class warning(Warning):
    pass

_psyco.NoLocalsWarning = warning

def warn(msg):
    from warnings import warn
    warn(msg, warning, stacklevel=2)

#
# Version checks
#
__version__ = 0x010600f0
if _psyco.PSYVER != __version__:
    raise error, "version mismatch between Psyco parts, reinstall it"

version_info = (__version__ >> 24,
                (__version__ >> 16) & 0xff,
                (__version__ >> 8) & 0xff,
                {0xa0: 'alpha',
                 0xb0: 'beta',
                 0xc0: 'candidate',
                 0xf0: 'final'}[__version__ & 0xf0],
                __version__ & 0xf)


VERSION_LIMITS = [0x02020200,   # 2.2.2
                  0x02030000,   # 2.3
                  0x02040000]   # 2.4

if ([v for v in VERSION_LIMITS if v <= sys.hexversion] !=
    [v for v in VERSION_LIMITS if v <= _psyco.PYVER  ]):
    if sys.hexversion < VERSION_LIMITS[0]:
        warn("Psyco requires Python version 2.2.2 or later")
    else:
        warn("Psyco version does not match Python version. "
             "Psyco must be updated or recompiled")


if hasattr(_psyco, 'ALL_CHECKS') and hasattr(_psyco, 'VERBOSE_LEVEL'):
    print >> sys.stderr, ('psyco: running in debugging mode on %s' %
                          _psyco.PROCESSOR)


###########################################################################
# sys._getframe() gives strange results on a mixed Psyco- and Python-style
# stack frame. Psyco provides a replacement that partially emulates Python
# frames from Psyco frames. The new sys._getframe() may return objects of
# a custom "Psyco frame" type, which is a subtype of the normal frame type.
#
# The same problems require some other built-in functions to be replaced
# as well. Note that the local variables are not available in any
# dictionary with Psyco.


class Frame:
    pass


class PythonFrame(Frame):

    def __init__(self, frame):
        self.__dict__.update({
            '_frame': frame,
            })

    def __getattr__(self, attr):
        if attr == 'f_back':
            try:
                result = embedframe(_psyco.getframe(self._frame))
            except ValueError:
                result = None
            except error:
                warn("f_back is skipping dead Psyco frames")
                result = self._frame.f_back
            self.__dict__['f_back'] = result
            return result
        else:
            return getattr(self._frame, attr)

    def __setattr__(self, attr, value):
        setattr(self._frame, attr, value)

    def __delattr__(self, attr):
        delattr(self._frame, attr)


class PsycoFrame(Frame):

    def __init__(self, tag):
        self.__dict__.update({
            '_tag'     : tag,
            'f_code'   : tag[0],
            'f_globals': tag[1],
            })

    def __getattr__(self, attr):
        if attr == 'f_back':
            try:
                result = embedframe(_psyco.getframe(self._tag))
            except ValueError:
                result = None
        elif attr == 'f_lineno':
            result = self.f_code.co_firstlineno  # better than nothing
        elif attr == 'f_builtins':
            result = self.f_globals['__builtins__']
        elif attr == 'f_restricted':
            result = self.f_builtins is not __builtins__
        elif attr == 'f_locals':
            raise AttributeError, ("local variables of functions run by Psyco "
                                   "cannot be accessed in any way, sorry")
        else:
            raise AttributeError, ("emulated Psyco frames have "
                                   "no '%s' attribute" % attr)
        self.__dict__[attr] = result
        return result

    def __setattr__(self, attr, value):
        raise AttributeError, "Psyco frame objects are read-only"

    def __delattr__(self, attr):
        if attr == 'f_trace':
            # for bdb which relies on CPython frames exhibiting a slightly
            # buggy behavior: you can 'del f.f_trace' as often as you like
            # even without having set it previously.
            return
        raise AttributeError, "Psyco frame objects are read-only"


def embedframe(result):
    if type(result) is type(()):
        return PsycoFrame(result)
    else:
        return PythonFrame(result)

def _getframe(depth=0):
    """Return a frame object from the call stack. This is a replacement for
sys._getframe() which is aware of Psyco frames.

The returned objects are instances of either PythonFrame or PsycoFrame
instead of being real Python-level frame object, so that they can emulate
the common attributes of frame objects.

The original sys._getframe() ignoring Psyco frames altogether is stored in
psyco._getrealframe(). See also psyco._getemulframe()."""
    # 'depth+1' to account for this _getframe() Python function
    return embedframe(_psyco.getframe(depth+1))

def _getemulframe(depth=0):
    """As _getframe(), but the returned objects are real Python frame objects
emulating Psyco frames. Some of their attributes can be wrong or missing,
however."""
    # 'depth+1' to account for this _getemulframe() Python function
    return _psyco.getframe(depth+1, 1)

def patch(name, module=__builtin__):
    f = getattr(_psyco, name)
    org = getattr(module, name)
    if org is not f:
        setattr(module, name, f)
        setattr(_psyco, 'original_' + name, org)

_getrealframe = sys._getframe
sys._getframe = _getframe
patch('globals')
patch('eval')
patch('execfile')
patch('locals')
patch('vars')
patch('dir')
patch('input')
_psyco.original_raw_input = raw_input
__builtin__.__in_psyco__ = 0==1   # False

if hasattr(_psyco, 'compact'):
    import kdictproxy
    _psyco.compactdictproxy = kdictproxy.compactdictproxy



import sys, re, cStringIO, os, dis, types, struct
import xam, psyco
from SimpleHTTPServer import SimpleHTTPRequestHandler, test

#
# Adapted from SimpleHTTPServer.py.
#

def show_vinfos(array, d, co=None, path=[]):
    text = "<ol>"
    for i in range(len(array)):
        vi = array[i]
        text += "<li>"
        if hasattr(co, 'co_code') and path == []:
            j = i - xam.LOC_LOCALS_PLUS
            if 0 <= j < len(co.co_varnames):
                text += "(%s):\t" % co.co_varnames[j]
        #if name is not None:
        #    text += "%s " % name
        if vi is None:
            text += "[NULL]"
        else:
            text += "[%x] %s" % (vi.addr, vi.gettext())
            if d.has_key(vi.addr):
                text += " (already seen above)"
            else:
                d[vi.addr] = 1
            if vi.array:
                text += show_vinfos(vi.array, d, co, path+[i])
        text += '</li>\n'
    text += '</ol>\n'
    return text

def summary_vinfos(array, d, path=[]):
    text = ''
    indent = '  ' * len(path)
    for i in range(len(array)):
        vi = array[i]
        text += indent
        if vi is None:
            text += "[NULL]\n"
        else:
            text += "%d. %s" % (i, vi.getsummarytext())
            if d.has_key(vi.addr):
                text += " (already seen above)"
            else:
                d[vi.addr] = 1
            text += '\n'
            if vi.array:
                text += summary_vinfos(vi.array, d, path+[i])
    return text

def find4(f, s4):
    result = []
    while 1:
        base = f.tell()
        buffer = f.read(8192)
        if not buffer:
            return result
        p = -1
        while 1:
            p = buffer.find(s4, p+1)
            if p<0:
                break
            if not (p&3):
                result.append(base + p)

re_codebuf = re.compile(r'[/]0x([0-9A-Fa-f]+)$')
re_proxy   = re.compile(r'[/]proxy(\d+)$')
re_summary = re.compile(r'[/]summary(\d+)$')
re_trace   = re.compile(r'[/]trace0x([0-9A-Fa-f]+)$')
re_traces  = re.compile(r'[/]traces0x([0-9A-Fa-f]+)$')
re_trlist  = re.compile(r'[/]trace0x([0-9A-Fa-f]+)[-]0x([0-9A-Fa-f]+)$')

##def cache_load(filename, cache={}):
##    try:
##        return cache[filename]
##    except KeyError:
##        data = {}
##        try:
##            f = execfile(filename, data)
##        except:
##            data = None
##        cache[filename] = data
##        return data

def cache_load(filename, codename, cache={}):
    try:
        modulecode = cache[filename]
    except KeyError:
        source = None
        try:
            f = open(filename, 'rU')
            source = f.read()
            f.close()
            modulecode = compile(source, filename, 'exec')
        except Exception, e:
            print repr(source)
            print '*** While loading %s:' % (filename,)
            import traceback
            traceback.print_exc()
            return None

    return recfindcode(modulecode, codename)

def recfindcode(code, codename):
    if code.co_name == codename:
        return code
    else:
        for c in code.co_consts:
            if type(c) is type(code):
                result = recfindcode(c, codename)
                if result:
                    return result
    return ''


class CodeBufHTTPHandler(SimpleHTTPRequestHandler):

    def symhtml(self, sym, addr, inbuf=None, lineaddr=None):
        text = xam.symtext(sym, addr, inbuf, lineaddr)
        if isinstance(sym, xam.CodeBuf):
            if addr == sym.addr:
                name = ''
            else:
                name = '#0x%x' % addr
            text = "<a href='/0x%x%s'>%s</a>" % (sym.addr, name, text)
        if addr == lineaddr:
            text += "\t<a href='/traces0x%x'>traces</a>" % addr
        return text

    def linehtml(self, line, addr):
        line = "<a name='0x%x'><strong>%s</strong></a>" % (addr, line)
        if addr in self.trace_addr:
            line = "<font color='#FF0000'>%s</font>" % line
            i = self.trace_addr.index(addr)
            if i == 0 and self.trace_prev is not None:
                line += ("\t<a href='/trace0x%x'>&lt;&lt;&lt;&lt;&lt;</a>" %
                         self.trace_prev)
            if i == len(self.trace_addr)-1 and self.trace_next is not None:
                line += ("\t<a href='/trace0x%x'>&gt;&gt;&gt;&gt;&gt;</a>" %
                         self.trace_next)
        return line

    def proxyhtml(self, proxy):
        return "<a href='/proxy%d'>(snapshot %s:%s)</a>\n" % (
            codebufs.index(proxy), proxy.co_name, proxy.get_next_instr())

    def htmlpage(self, title, data):
        return ('<html><head><title>%s</title></head>\n'  % title
                + '<body><h1>%s</h1>\n'               % title
              # + '<hr>\n'
                + data
                + '<hr></body>\n')

    def bufferpage(self, codebuf):
        rev = {}
        for o, c in codebuf.reverse_lookup:
            if c is not codebuf:
                rev[c] = rev.get(c,0) + 1
        if rev:
            data = '<p>Other code buffers pointing to this one:</p><ul>\n'
            for c in codebufs:  # display them in original load order
                if rev.has_key(c):
                    if rev[c] == 1:
                        extra = ''
                    else:
                        extra = '\t(%d times)' % rev[c]
                    data += '<li>%s\t(%d bytes)%s</li>\n' %  \
                            (self.symhtml(c, c.addr),
                             len(c.data),
                             extra)
            data += '</ul>\n'
        else:
            data = '<p>No other code buffer points to this one.</p>\n'
        data += '<hr>\n'
        data += '<pre>%s</pre>\n' % codebuf.disassemble(self.symhtml,
                                                        self.linehtml,
                                                        self.proxyhtml)
        data += "<br><a href='/'>Back to the list of code objects</a>\n"
        if codebuf.co_name:
            data = '<p>Code object %s from file %s, at position %s</p>%s' % (
                codebuf.co_name, codebuf.co_filename, codebuf.get_next_instr(),
                data)
        return data

    def try_hard_to_name(self, addr):
        def result(codebuf):
            return '%s:%s:%s' % (codebuf.co_filename, codebuf.co_name,
                                 codebuf.get_next_instr())
        codebuf = xam.codeat(addr)
        if codebuf is not None:
            codemap = codebuf.codemap
            proxylist = []
            for lineaddr in range(addr, codebuf.addr-1, -1):
                if codemap.has_key(lineaddr):
                    for proxy in codemap[lineaddr]:
                        if proxy.co_name:
                            return result(proxy)
        if codebuf.co_name:
            return result(codebuf)
        else:
            return '?'

    def send_head(self):
        global codebufs # CT
        self.trace_prev = None
        self.trace_next = None
        self.trace_addr = ()
        
        if self.path == '/' or self.path == '/all':
            all = self.path == '/all'
            if all:
                title = 'List of ALL code objects'
            else:
                title = 'List of all named code objects'
            data = ['<ul>']
            named = 0
            proxies = 0
            for codebuf in codebufs:
                if codebuf.data and codebuf.co_name:
                    named += 1
                else:
                    if not codebuf.data:
                        proxies += 1
                    if not all:
                        continue
                data.append('<li>%s:\t%s:\t%s:\t%s\t(%d bytes)</li>\n' % (
                    codebuf.co_filename, codebuf.co_name,
                    codebuf.get_next_instr(),
                    self.symhtml(codebuf, codebuf.addr),
                    len(codebuf.data)))
            data.append('</ul>\n')
            data.append('<br><a href="/">%d named buffers</a>; ' % named +
                     '<a href="/all">%d buffers in total</a>, ' % len(codebufs) +
                     'including %d proxies' % proxies)
            data = ''.join(data)
            return self.donepage(title, data)

        match = re_codebuf.match(self.path)
        if match:
            addr = long(match.group(1), 16)
            codebuf = xam.codeat(addr)
            if not codebuf:
                self.send_error(404, "No code buffer at this address")
                return None
            if codebuf.addr != addr:
                self.trace_addr = [addr]
            title = '%s code buffer at 0x%x' % (codebuf.mode.capitalize(),
                                                codebuf.addr)
            data = self.bufferpage(codebuf)
            return self.donepage(title, data)

        match = re_trace.match(self.path)
        if match:
            tracepos = int(match.group(1), 16)
            f = open(tracefilename, 'rb')
            try:
                def traceat(p, f=f):
                    f.seek(p)
                    data = f.read(4)
                    if len(data) == 4:
                        addr, = struct.unpack('L', data)
                        return addr
                    else:
                        raise IOError
                addr = traceat(tracepos)
                codebuf = xam.codeat(addr)
                if not codebuf:
                    self.send_error(404, "No code buffer at 0x%x" % addr)
                    return None
                start = codebuf.addr
                end = start + len(codebuf.data)
                while tracepos > 0:
                    addr1 = traceat(tracepos-4)
                    if not (start <= addr1 < addr):
                        break
                    tracepos -= 4
                    addr = addr1
                self.trace_prev = tracepos-4
                self.trace_addr = []
                while 1:
                    self.trace_addr.append(addr)
                    addr1 = traceat(tracepos+4)
                    if not (addr < addr1 < end):
                        break
                    tracepos += 4
                    addr = addr1
                self.trace_next = tracepos+4
            finally:
                f.close()
            title = '%s code buffer at 0x%x' % (codebuf.mode.capitalize(),
                                                codebuf.addr)
            data = self.bufferpage(codebuf)
            return self.donepage(title, data)

        match = re_traces.match(self.path)
        if match:
            traceaddr = long(match.group(1), 16)
            f = open(tracefilename, 'rb')
            plist = find4(f, struct.pack('L', traceaddr))
            f.close()
            title = 'Traces through 0x%x' % traceaddr
            data = ['<ul>']
            for p in plist:
                data.append("<li><a href='/trace0x%x'>0x%x</a>\n" % (p, p))
            data.append('</ul>')
            data = ''.join(data)
            return self.donepage(title, data)

        match = re_trlist.match(self.path)
        if match:
            start = int(match.group(1), 16)
            end   = int(match.group(2), 16)
            title = 'Traces timed 0x%x to 0x%x' % (start, end)
            data = ["<p><a href='/trace0x%x-0x%x'>&lt;&lt;&lt;&lt;</a></p>\n" %
                    (start-(end-start), start),
                    '<ul>']
            f = open(tracefilename, 'rb')
            f.seek(start)
            prevname = None
            for p in range(start, end, 4):
                addr, = struct.unpack('L', f.read(4))
                s = self.try_hard_to_name(addr)
                if s == prevname:
                    continue
                data.append("<li><a href='/trace0x%x'>0x%x: %s</a>\n" % (p,p,s))
                prevname = s
            data.append('</ul>')
            data.append(
                "<p><a href='/trace0x%x-0x%x'>&gt;&gt;&gt;&gt;</a></p>\n" %
                (end, end+(end-start)))
            f.close()
            data = ''.join(data)
            return self.donepage(title, data)

        match = re_proxy.match(self.path)
        if match:
            title = 'Snapshot'
            n = int(match.group(1))
            proxy = codebufs[n]
            for n1 in xrange(n-1, -1, -1):
                pprev = codebufs[n1]
                if (pprev.nextinstr == proxy.nextinstr and
                    pprev.co_name == proxy.co_name and
                    pprev.co_filename == proxy.co_filename):
                    pprev = n1
                    break
            else:
                pprev = None
            for n1 in xrange(n+1, len(codebufs)):
                pnext = codebufs[n1]
                if (pnext.nextinstr == proxy.nextinstr and
                    pnext.co_name == proxy.co_name and
                    pnext.co_filename == proxy.co_filename):
                    pnext = n1
                    break
            else:
                pnext = None
            filename = os.path.join(DIRECTORY, proxy.co_filename)
            co = cache_load(filename, proxy.co_name)
            data = '<p>PsycoObject structure at this point:'
            data += '&nbsp;' * 20
            data += '['
            data += '&nbsp;&nbsp;<a href="summary%d">summary</a>&nbsp;&nbsp;' % n
            if pprev is not None or pnext is not None:
                if pprev is not None:
                    data += '&nbsp;&nbsp;<a href="proxy%d">&lt;&lt;&lt; previous</a>&nbsp;&nbsp;' % pprev
                if pnext is not None:
                    data += '&nbsp;&nbsp;<a href="proxy%d">next &gt;&gt;&gt;</a>&nbsp;&nbsp;' % pnext
            data += ']'
            data += '</p>\n'
            data += show_vinfos(proxy.vlocals, {}, co)
            data += '<hr><p>Disassembly of %s:%s:%s:</p>\n' % (
                proxy.co_filename, proxy.co_name, proxy.get_next_instr())
            if co is None: #moduledata is None:
                txt = "(exception while loading the file '%s')\n" % (
                    filename)
            else:
                if not hasattr(co, 'co_code'):
                    txt = "(no function object '%s' in file '%s')\n" % (
                        proxy.co_name, filename)
                else:
                    txt = cStringIO.StringIO()
                    oldstdout = sys.stdout
                    try:
                        sys.stdout = txt
                        dis.disassemble(co, proxy.get_next_instr())
                    finally:
                        sys.stdout = oldstdout
                    txt = txt.getvalue()
            data += '<pre>%s</pre>\n' % txt
            data += "<br><a href='/0x%x'>Back</a>\n" % proxy.addr
            return self.donepage(title, data)

        match = re_summary.match(self.path)
        if match:
            n = int(match.group(1))
            proxy = codebufs[n]
            data = summary_vinfos(proxy.vlocals, {})
            f = cStringIO.StringIO(data)
            self.send_response(200)
            self.send_header("Content-type", "text/plain")
            self.end_headers()
            return f

        if self.path == '/checkall':
            for codebuf in codebufs:
                codebuf.cache_text
            f = cStringIO.StringIO('done')
            self.send_response(200)
            self.send_header("Content-type", "text/plain")
            self.end_headers()
            return f

        ## CT: simple reload feature
        if self.path == "/reload":
            codebufs = xam.readdump(FILENAME)
            self.path = "/all"
            return self.send_head()
        
        self.send_error(404, "Invalid path")
        return None

    def donepage(self, title, data):
        f = cStringIO.StringIO(self.htmlpage(title, data))
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        return f


if __name__ == '__main__':
    if len(sys.argv) <= 1:
        print "Usage: python httpxam.py <directory>"
        print "  psyco.dump and any .py files containing code objects"
        print "  are loaded from the <directory>."
        sys.exit(1)
    DIRECTORY = sys.argv[1]
    del sys.argv[1]
    filename = os.path.join(DIRECTORY, 'psyco.dump')
    if not os.path.isfile(filename) and os.path.isfile(DIRECTORY):
        filename = DIRECTORY
        DIRECTORY = os.path.dirname(DIRECTORY)
    tracefilename = os.path.join(DIRECTORY, 'psyco.trace')
    codebufs = xam.readdump(filename)
    FILENAME = filename # CT hack
    test(CodeBufHTTPHandler)



from __future__ import generators
import os
import py   # the py lib, see http://codespeak.net/py

def psycofiles():
    path = py.path.svnwc(os.pardir)
    for p in path.visit(lambda x: x.check(versioned=1)):
        if p.check(dir=1):
            print p
        else:
            yield p.relto(path)

def generate():
    filename = os.path.join('..', 'MANIFEST')
    print 'Rebuilding %s...' % filename
    lst = list(psycofiles())
    lst.sort()
    f = open(filename, 'w')
    for filename in lst:
        print >> f, filename
    f.close()

if __name__ == '__main__':
    generate()



from xam import *
import httpxam


def load_vi_array(dumpfile, d):
    match = re_int.match(dumpfile.readline())
    assert match
    count = int(match.group(1))
    a = []
    for i in range(count):
        line = dumpfile.readline()
        match = re_int.match(line)
        assert match
        addr = long(match.group(1))
        if d.has_key(addr):
            vi = d[addr]
        else:
            line = dumpfile.readline()
            match = re_ctvinfo.match(line)
            if match:
                vi = CompileTimeVInfo(int(match.group(1)),
                                      long(match.group(2)))
            else:
                match = re_rtvinfo.match(line)
                if match:
                    vi = RunTimeVInfo(long(match.group(1)))
                else:
                    match = re_vtvinfo.match(line)
                    assert match
                    vi = VirtualTimeVInfo(long(match.group(1), 16))
            d[addr] = vi
            vi.addr = addr
            vi.array = load_vi_array(dumpfile, d)
        a.append(vi)
    a.reverse()
    return a

def main(dumpfile):
    import os, tempfile
    array = load_vi_array(dumpfile, {0: None})
    text = httpxam.show_vinfos(array, {})
    if os.fork() == 0:
        TMP = tempfile.mktemp('.html')
        g = open(TMP, 'w')
        g.write('<html><head></head><body>\n')
        g.write(text)
        g.write('</body>\n')
        g.close()
        try:
            os.system('xterm -e lynx -force_html %s' % TMP)
        finally:
            os.unlink(TMP)

if __name__ == '__main__':
    import sys
    main(sys.stdin)



###########################################################################
#
#  Support code for the 'psyco.compact' type.

from __future__ import generators

try:
    from UserDict import DictMixin
except ImportError:

    # backported from Python 2.3 to Python 2.2
    class DictMixin:
        # Mixin defining all dictionary methods for classes that already have
        # a minimum dictionary interface including getitem, setitem, delitem,
        # and keys. Without knowledge of the subclass constructor, the mixin
        # does not define __init__() or copy().  In addition to the four base
        # methods, progressively more efficiency comes with defining
        # __contains__(), __iter__(), and iteritems().

        # second level definitions support higher levels
        def __iter__(self):
            for k in self.keys():
                yield k
        def has_key(self, key):
            try:
                value = self[key]
            except KeyError:
                return False
            return True
        def __contains__(self, key):
            return self.has_key(key)

        # third level takes advantage of second level definitions
        def iteritems(self):
            for k in self:
                yield (k, self[k])
        def iterkeys(self):
            return self.__iter__()

        # fourth level uses definitions from lower levels
        def itervalues(self):
            for _, v in self.iteritems():
                yield v
        def values(self):
            return [v for _, v in self.iteritems()]
        def items(self):
            return list(self.iteritems())
        def clear(self):
            for key in self.keys():
                del self[key]
        def setdefault(self, key, default):
            try:
                return self[key]
            except KeyError:
                self[key] = default
            return default
        def pop(self, key, *args):
            if len(args) > 1:
                raise TypeError, "pop expected at most 2 arguments, got "\
                                  + repr(1 + len(args))
            try:
                value = self[key]
            except KeyError:
                if args:
                    return args[0]
                raise
            del self[key]
            return value
        def popitem(self):
            try:
                k, v = self.iteritems().next()
            except StopIteration:
                raise KeyError, 'container is empty'
            del self[k]
            return (k, v)
        def update(self, other):
            # Make progressively weaker assumptions about "other"
            if hasattr(other, 'iteritems'):  # iteritems saves memory and lookups
                for k, v in other.iteritems():
                    self[k] = v
            elif hasattr(other, '__iter__'): # iter saves memory
                for k in other:
                    self[k] = other[k]
            else:
                for k in other.keys():
                    self[k] = other[k]
        def get(self, key, default=None):
            try:
                return self[key]
            except KeyError:
                return default
        def __repr__(self):
            return repr(dict(self.iteritems()))
        def __cmp__(self, other):
            if other is None:
                return 1
            if isinstance(other, DictMixin):
                other = dict(other.iteritems())
            return cmp(dict(self.iteritems()), other)
        def __len__(self):
            return len(self.keys())

###########################################################################

from _psyco import compact


class compactdictproxy(DictMixin):

    def __init__(self, ko):
        self._ko = ko    # compact object of which 'self' is the dict

    def __getitem__(self, key):
        return compact.__getslot__(self._ko, key)

    def __setitem__(self, key, value):
        compact.__setslot__(self._ko, key, value)

    def __delitem__(self, key):
        compact.__delslot__(self._ko, key)

    def keys(self):
        return compact.__members__.__get__(self._ko)

    def clear(self):
        keys = self.keys()
        keys.reverse()
        for key in keys:
            del self[key]

    def __repr__(self):
        keys = ', '.join(self.keys())
        return '<compactdictproxy object {%s}>' % (keys,)



#! /usr/bin/env python

"""List of source files of Psyco, the Python specializing compiler

This script can be used to rebuild various headers and the MANIFEST."""


class Source:
    def __init__(self, filename, initname=None):
        self.filename = filename
        self.initname = initname

class Object(Source):
    def __init__(self, name, has_init=1):
        if has_init:
            initname = 'psy_%s_init' % name
        else:
            initname = None
        Source.__init__(self, 'Objects/p%s.c' % name, initname)

class Module(Source):
    def __init__(self, name):
        Source.__init__(self, 'Modules/p%s.c' % name, 'psyco_init%s' % name)


SRC = [
    Source('dispatcher.c'),
    Source('vcompiler.c',	'psyco_compiler_init'),
    Source('psyco.c'),
    Source('platform.c'),
    Source('psyfunc.c'),
    Source('stats.c',		'psyco_stats_init'),
    Source('profile.c',         'psyco_profile_init'),
    Source('cstruct.c',		'psyco_cstruct_init'),
    Source('alarm.c',		'psyco_alarm_init'),
    Source('codemanager.c'),
    Source('codegen.c',		'psyco_codegen_init'),
    Source('mergepoints.c'),
    Source('linuxmemchk.c'),
    Source('Python/pycompiler.c',	'psyco_pycompiler_init'),
    Source('Python/frames.c',		'psyco_frames_init'),
    Source('Python/pbltinmodule.c',	'psyco_bltinmodule_init'),

    Object('object'),
    Object('abstract', 0),
    Object('boolobject'),
    Object('classobject'),
    Object('descrobject'),
    Object('dictobject'),
    Object('floatobject'),
    Object('funcobject'),
    Object('intobject'),
    Object('iterobject'),
    Object('listobject'),
    Object('longobject'),
    Object('methodobject'),
    Object('rangeobject'),
    Object('stringobject'),
    Object('structmember', 0),
    Object('tupleobject'),
    Object('typeobject'),
    Source('Objects/compactobject.c',  'psyco_compact_init'),
    Object('compactobject'),

    Module('array'),
    Module('math'),
    Module('psyco'),
    ]

PROCESSOR_SRC = {
    'i386': [
        Source('iprocessor.c',	'psyco_processor_init'),
        Source('idispatcher.c'),
        Source('iencoding.c'),
        Source('ipyencoding.c'),
        ],
    'ivm': [
        Source('iprocessor.c'),
        Source('idispatcher.c'),
        Source('iencoding.c'),
        Source('ipyencoding.c'),
        Source('ivm-insns.c'),
        ],
    }

MAINFILE = 'psyco.c'
PLATFILE = 'platform.c'


def generate(processor=None):
    if processor:
        filename = '%s/iinitialize.h' % processor
        src = PROCESSOR_SRC[processor]
    else:
        filename = 'initialize.h'
        src = SRC
    header = """\
 /***************************************************************/
/***          Automatically generated support file             ***/
 /***************************************************************/

 /* This file is automatically generated by 'files.py'.
    DO NOT MODIFY. Changes will be overwritten ! */

"""
    print 'Rebuilding %s...' % filename
    f = open(filename, 'w')
    print >> f, header
    if not processor:
        print >> f, ' /* Including this file results in all headers Objects/xxx.h'
        print >> f, '    being included, so that it has roughly the same result'
        print >> f, '    for Psyco as a "#include <Python.h>" has for Python:'
        print >> f, '    including all headers extension modules generally need.'
        print >> f
        print >> f, '    This file is moreover used internally by psyco.c. */'
        print >> f
        print >> f
        print >> f, '#ifndef PSYCO_INITIALIZATION'
        print >> f
        for s in src:
            if isinstance(s, Object):
                if processor or s.filename != PLATFILE:
                    assert s.filename.endswith(".c")
                    print >> f, '# include "%s"' % (s.filename[:-2] + ".h")
        print >> f
        print >> f, '#else /* if PSYCO_INITIALIZATION */'
        print >> f, '# undef PSYCO_INITIALIZATION'
        print >> f
        print >> f, '#include <iinitialize.h>  /* processor-specific initialization */'
        print >> f
    print >> f, '  /* internal part for psyco.c */'
    print >> f, '#if ALL_STATIC'
    for s in src:
        if processor or s.filename not in (MAINFILE, PLATFILE):
            print >> f, '# include "%s"' % s.filename
    print >> f, '#else /* if !ALL_STATIC */'
    for s in src:
        if s.initname:
            print >> f, '  EXTERNFN void %s(void);\t/* %s */' % (s.initname, s.filename)
    print >> f, '#endif /* !ALL_STATIC */'
    print >> f
    if processor:
        print >> f, 'PSY_INLINE void initialize_processor_files(void) {'
    else:
        print >> f, 'PSY_INLINE void initialize_all_files(void) {'
        print >> f, '  initialize_processor_files();'
    for s in src:
        if s.initname:
            print >> f, '  %s();\t/* %s */' % (s.initname, s.filename)
    print >> f, '}'
    if not processor:
        print >> f
        print >> f, '#endif /* PSYCO_INITIALIZATION */'
    f.close()

def main():
    generate()
    for processor in PROCESSOR_SRC.keys():
        generate(processor)
    
    import os, sys; sys.path.insert(0, os.path.join(os.pardir, 'py-utils'))
    import manifest
    manifest.generate()

if __name__ == '__main__':
    main()



from __future__ import nested_scopes
import os, sys, re, htmlentitydefs, struct, bisect
__metaclass__ = type

tmpfile = '~tmpfile.tmp'

# the disassembler to use. 'objdump' writes GNU-style instructions.
# 'ndisasm' uses Intel syntax.

objdump = 'objdump -b binary -m i386 --adjust-vma=%(origin)d -D %(file)s'
if sys.platform == "win32":
    try:
        from xam import __file__ as _xamfile
    except ImportError:
        raise ImportError, "could not import xam module"
    _win32_path = os.path.join(os.path.split(_xamfile)[0], "win32")
    objdump = os.path.join(_win32_path, objdump)
    _objdumpexe = objdump.split()[0]+".exe"
    # test whether it works:
    if os.system(_objdumpexe + " -v"):
        raise IOError, "file %s and cygwin1.dll must exist" % _objdumpexe
#objdump = 'ndisasm -o %(origin)d -u %(file)s'

# the files from which symbols are loaded.
# the order and number of files must match
# psyco_dump_code_buffers() in psyco.c.
symbolfiles = [sys.executable]
try:
    from psyco import _psyco
    symbolfiles.append(_psyco.__file__)
except ImportError:
    pass

# the program that lists symbols, and the output it gives
symbollister = 'nm %s'
re_symbolentry = re.compile(r'([0-9a-fA-F]+)\s\w\s(.*)')

if sys.platform == "win32":
    # no way to get full info into the executables by
    # VC7. /PDB:NONE no longer supported.
    # so we have to read the map files.
    if sys.executable.lower().endswith("_d.exe"):
        _mapfiles = ("python23_d.map", "_psyco_d.map")
    else:
        _mapfiles = ("python23.map", "_psyco.map")
    symbolfiles = [os.path.join(_win32_path, x) for x in _mapfiles]
    for _filepath in symbolfiles:
        if not os.path.exists(_filepath):
            raise IOError, "please make sure that '%s' exists" % _filepath
        
    class symbollister:
        def __init__(self, filename):
            self.file = file(filename)
            self.generator = self._readline()

        def _readline(self):
            for line in self.file:
                #  0001:000661e0       _PyEval_CallFunction       1e0671e0 f   modsupport.obj
                #  0003:0000e770       _PyClass_Type              1e0d8770     classobject.obj
                pieces = line.split()
                if len(pieces) == 5:
                    colonadr, name, adr, dummy, obj = pieces
                elif len(pieces) == 4:
                    colonadr, name, adr, obj = pieces
                    dummy = "d"
                else:
                    continue
                if colonadr.count(":") == 1 and obj.endswith(".obj"):
                    yield "%s %s %s\n" % (adr, dummy, name[1:])

        def readline(self):
            try:
                return self.generator.next()
            except StopIteration:
                return ""

        def close(self):
            self.file.close()
            
        def __iter__(self):
            return self.generator


re_addr = re.compile(r'[\s,$]0x([0-9a-fA-F]+)')
re_lineaddr = re.compile(r'\s*0?x?([0-9a-fA-F]+)')


symbols = {}
#rawtargets = {}
codeboundary = []

try:
    from xamsupport import any_pointer
except ImportError:
    def any_pointer(addr0, data, start, end, unpack=struct.unpack):
        for i in range(4, len(data)+1):
            offset, = unpack('l', data[i-4:i])
            if start <= addr0+i+offset < end or start <= offset < end:
                return 1
        return 0

def machine_code_dump(data, originaddr, format):
    if format == 'ivm':
        import ivmdump
        result = ivmdump.dump(data, originaddr)
    elif format == 'i386':
        f = open(tmpfile, 'wb')
        f.write(data)
        f.close()
        try:
            g = os.popen(objdump % {'file': tmpfile, 'origin': originaddr}, 'r')
            result = g.readlines()
            g.close()
        finally:
            os.unlink(tmpfile)
    return result

def load_symbol_file(filename, symb1, addr1):
    d = {}
    if type(symbollister) is str:
        g = os.popen(symbollister % filename, "r")
    else:
        g = symbollister(filename)
    while 1:
        line = g.readline()
        if not line:
            break
        match = re_symbolentry.match(line)
        if match:
            d[match.group(2)] = long(match.group(1), 16)
    g.close()
    if d.has_key(symb1):
        delta = addr1 - d[symb1]
    else:
        delta = 0
        print >> sys.stderr,"Warning: no symbol '%s' in '%s'" % (symb1, filename)
    for key, value in d.items():
        symbols[value + delta] = key


def symtext(sym, addr, inbuf=None, lineaddr=None):
    if isinstance(sym, CodeBuf):
        if sym is inbuf:
            name = 'top'
        else:
            name = '%s codebuf 0x%x' % (sym.mode, sym.addr)
        if addr > sym.addr:
            name += ' + %d' % (addr-sym.addr)
        return name
    else:
        return sym

revmap = {}
for key, value in htmlentitydefs.entitydefs.items():
    if type(value) is type(' '):
        revmap[value] = '&%s;' % key

def htmlquote(text):
    return ''.join([revmap.get(c,c) for c in text])

def lineaddresses(line):
    result = []
    i = 0
    while 1:
        match = re_addr.search(line, i)
        if not match:
            break
        i = match.end()
        addr = long(match.group(1), 16)
        result.append(addr)
    return result

def codeat(addr):
    i = bisect.bisect(codeboundary, (addr, None))
    if i>0:
        addrend, codebuf = codeboundary[i-1]
        if isinstance(codebuf, CodeBuf):
            return codebuf


re_int = re.compile(r"(\-?\d+)$")
re_ctvinfo = re.compile(r"ct (\d+) (\-?\d+)$")
re_rtvinfo = re.compile(r"rt (\-?\d+)$")
re_vtvinfo = re.compile(r"vt 0x([0-9a-fA-F]+)$")

LOC_LOCALS_PLUS = 3

class CodeBuf:
    __slots__ = ['mode', 'co_filename', 'co_name', 'nextinstr', 'addr',
                 'stackdepth', 'specdict', 'data', 'cache_text',
                 'disass_text', 'reverse_lookup', 'vlocals',
                 'complete_list', 'dumpfile', 'vlocalsofs', 'codemap']
    machine_code_format = '?'
    
    def __init__(self, mode, co_filename, co_name, nextinstr,
                 addr, stackdepth):
        self.mode = mode
        self.co_filename = co_filename
        self.co_name = co_name
        self.nextinstr = nextinstr
        self.addr = addr
        #self.data = data
        self.stackdepth = stackdepth
        #self.reverse_lookup = []  # list of (offset, codebuf pointing there)
        self.specdict = []
        if self.mode != "proxy":
            codeboundary.append((self.addr-0.5, self))
        else:
            self.data = ""
        #for i in range(4, len(data)+1):
        #    offset, = struct.unpack('l', data[i-4:i])
        #    rawtargets.setdefault(addr+i+offset, {})[self] = 1

    def getboundary(self):
        i = bisect.bisect(codeboundary, (self.addr-0.5, self))
        prev = codeboundary[i-1][1]
        next = codeboundary[i][1]
        #while not isinstance(next, BigBuffer) and next.addr == self.addr:
        #    i = i + 1
        #    next = codeboundary[i][1]
        while not isinstance(codeboundary[i][1], BigBuffer):
            i = i + 1
        bigbuf = codeboundary[i][1]
        return prev, next, bigbuf

    def splitheader(self):
        data = self.data
        addr = self.addr
        k = 0
        while data[k:k+1] == '\xCC':
            k = k + 1
        if data[k:k+4] == '\x66\x66\x66\x66':
            # detected a rt_local_buf_t structure
            next, key = struct.unpack('LL', data[k+4:k+12])
            data = data[k+12:]
            addr += k+12
        else:
            next = key = None
        return data, addr, next, key

    def __getattr__(self, attr):
        if attr == 'data':
            prev, next, bigbuf = self.getboundary()
            assert prev is self
            self.data = data = bigbuf.load(self.addr, next.addr)
            return data
        if attr == 'cache_text':
            # produce the disassembly listing
            data, addr, next, key = self.splitheader()
            self.cache_text = []
            if key is not None:
                self.cache_text.append(
                    'Created by promotion of the value 0x%x\n' % key)
            if next is not None:
                self.cache_text.append(
                    'Next promoted value at buffer 0x%x\n' % next)
            self.cache_text += machine_code_dump(data, addr,
                                                 CodeBuf.machine_code_format)
            return self.cache_text
        if attr == 'disass_text':
            txt = self.cache_text
            if self.specdict:
                txt.append('\n')
                txt.append("'do_promotion' dictionary:\n")
                for key, value in self.specdict:
                    txt.append('.\t%s:\t\t\n' % htmlquote(key))
                    txt.append('.\t\t0x%x\t\t\n' % value)
            self.disass_text = txt
            return txt
        if attr == 'reverse_lookup':
            # 'reverse_lookup' is a list of (offset, codebuf pointing there)
            self.reverse_lookup = []
            start = self.addr
            end = start + len(self.data)
            for codebuf in self.complete_list:
                if any_pointer(codebuf.addr, codebuf.data, start, end):
                    for line in codebuf.disass_text:
                        for addr in lineaddresses(line):
                            if start <= addr < end:
                                self.reverse_lookup.append((addr-start, codebuf))
            return self.reverse_lookup
        if attr == 'vlocals':
            self.dumpfile.seek(self.vlocalsofs)
            self.vlocals = self.load_vi_array({0: None})
            return self.vlocals
        raise AttributeError, attr

    def load_vi_array(self, d):
        dumpfile = self.dumpfile
        match = re_int.match(dumpfile.readline())
        assert match
        count = int(match.group(1))
        a = []
        for i in range(count):
            line = dumpfile.readline()
            match = re_int.match(line)
            assert match
            addr = long(match.group(1))
            if d.has_key(addr):
                vi = d[addr]
            else:
                line = dumpfile.readline()
                match = re_ctvinfo.match(line)
                if match:
                    vi = CompileTimeVInfo(int(match.group(1)),
                                          long(match.group(2)))
                else:
                    match = re_rtvinfo.match(line)
                    if match:
                        vi = RunTimeVInfo(long(match.group(1)), self.stackdepth)
                    else:
                        match = re_vtvinfo.match(line)
                        assert match
                        vi = VirtualTimeVInfo(long(match.group(1), 16))
                d[addr] = vi
                vi.addr = addr
                vi.array = self.load_vi_array(d)
            a.append(vi)
        a.reverse()
        return a

    def get_next_instr(self):
        if self.nextinstr >= 0:
            return self.nextinstr

    def spec_dict(self, key, value):
        self.specdict.append((key, value))
        #rawtargets.setdefault(value, {})[self] = 1
        try:
            del self.disass_text
        except:
            pass
        try:
            del self.reverse_lookup
        except:
            pass
    
##    def build_reverse_lookup(self):
##        for line in self.disass_text:
##            for addr in lineaddresses(line):
##                sym = symbols.get(addr)
##                if isinstance(sym, CodeBuf):
##                    sym.reverse_lookup.append((addr-sym.addr, self))
    
    def disassemble(self, symtext=symtext, linetext=None, snapshot=None):
        seen = {}
        data = []
        for line in self.disass_text:
            if line.endswith('\n'):
                line = line[:-1]
            match = re_lineaddr.match(line)
            if match:
                lineaddr = long(match.group(1), 16)
                if not seen.has_key(lineaddr):
                    if self.codemap.has_key(lineaddr) and snapshot:
                        for proxy in self.codemap[lineaddr]:
                            data.append(snapshot(proxy))
                    seen[lineaddr] = 1
                ofs = lineaddr - self.addr
                sources = [c for o, c in self.reverse_lookup if o == ofs]
                if sources and linetext:
                    line = linetext(line, lineaddr)
                if sources != [self]*len(sources):
                    data.append('\n')
            else:
                lineaddr = None
            for addr in lineaddresses(line):
                sym = symbols.get(addr) or codeat(addr)
                if sym:
                    line = '%s\t(%s)' % (line, symtext(sym, addr, self,lineaddr))
                    break
            data.append(line + '\n')
        return ''.join(data)


class BigBuffer:
    __slots__ = ['file', 'offset', 'start', 'length', 'addr', 'priority']
    def __init__(self, file, start, length):
        #if sys.stderr.softspace:
        #    print >> sys.stderr
        #print >> sys.stderr, 'BigBuffer:', hex(start), hex(start+length),
        #print >> sys.stderr, '(%d)' % length
        self.file = file
        self.offset = file.tell()
        self.start = start
        self.length = length
        self.addr = start + length   # end address
        self.priority = -len(codeboundary)
        codeboundary.append((self.addr-0.25, self))
        file.seek(self.length, 1)
    def load(self, begin, end):
        assert self.start <= begin <= self.addr, \
               (hex(self.start), hex(begin), hex(end), hex(self.addr))
        self.file.seek(self.offset + (begin-self.start))
        return self.file.read(min(self.addr, end) - begin)


class VInfo:
    __slots__ = ['addr', 'array']

class CompileTimeVInfo(VInfo):
    __slots__ = ['flags', 'value']
    def __init__(self, flags, value):
        self.flags = flags
        self.value = value
    def gettext(self):
        text = "Compile-time value 0x%x" % self.value
        if self.flags & 1:
            text += ", fixed"
        if self.flags & 2:
            text += ", reference"
        return text
    def getsummarytext(self):
        text = "Compile-time"
        if self.flags & 1:
            text += " fixed"
        text += " 0x%x" % self.value
        return text

class RunTimeVInfo(VInfo):
    __slots__ = ['source', 'stackdepth']
    REG_NAMES = ["eax", "ecx", "edx", "ebx", "esp", "ebp", "esi", "edi"]
    def __init__(self, source, stackdepth=None):
        self.source = source
        self.stackdepth = stackdepth
    def gettext(self):
        text = "Run-time source,"
        reg = self.source >> 28
        stack = self.source & 0x03FFFFFC
        if CodeBuf.machine_code_format == 'ivm':
            if reg:
                text += " in a register ??????"
            if not stack:
                text += " not in stack ??????"
            else:
                text += " in stack [%d] or from top #%d" % (
                    (self.stackdepth-stack)/4,
                    stack/4)
        else:
            if 0 <= reg < 8:
                text += " in register %s" % self.REG_NAMES[reg].upper()
                if stack:
                    text += " and"
            if stack:
                if self.stackdepth is None:
                    sd = ""
                else:
                    sd = "[ESP+0x%x] or " % (self.stackdepth - stack)
                text += " in stack %sfrom top %d" % (sd, stack)
        if not (self.source & 0x08000000):
            text += " holding a reference"
        if self.source & 0x04000000:
            text += " >=0"
        return text
    def getsummarytext(self):
        return "Run-time"

class VirtualTimeVInfo(VInfo):
    __slots__ = ['vs']
    def __init__(self, vs):
        self.vs = vs
    def gettext(self):
        return "Virtual-time source (%x)" % self.vs
    def getsummarytext(self):
        return "Virtual-time (%x)" % self.vs

def readdump(filename = 'psyco.dump'):
    del codeboundary[:]
    re_header = re.compile(r"Psyco dump [[](\w+?)[]]")
    re_symb1 = re.compile(r"(\w+?)[:]\s0x([0-9a-fA-F]+)")
    re_codebuf = re.compile(r"CodeBufferObject 0x([0-9a-fA-F]+) (\-?\d+) \'(.*?)\' \'(.*?)\' (\-?\d+) \'(.*?)\'$")
    re_specdict = re.compile(r"spec_dict 0x([0-9a-fA-F]+)")
    re_vinfo_array = re.compile(r"vinfo_array")
    re_spec1 = re.compile(r"0x([0-9a-fA-F]+)\s(.*)$")
    re_bigbuffer = re.compile(r"BigBuffer 0x([0-9a-fA-F]+) (\d+)$")
    
    codebufs = []
    dumpfile = open(filename, 'rb')
    match = re_header.match(dumpfile.readline())
    if not match:
        raise ValueError, "'%s' does not look like a Psyco dump" % filename
    CodeBuf.machine_code_format = match.group(1)
    
    bufcount, = struct.unpack("i", dumpfile.read(4))
    buftable = list(struct.unpack("l"*bufcount, dumpfile.read(4*bufcount)))
    buftable.reverse()
    if buftable:
        filesize = buftable[-1]
    else:
        filesize = sys.maxint
    filesize *= 1.0
    nextp = 0.1
    cbsortedsize = 0
    for filename in symbolfiles:
        line = dumpfile.readline()
        match = re_symb1.match(line)
        assert match
        load_symbol_file(filename, match.group(1), long(match.group(2), 16))
    while 1:
        line = dumpfile.readline()
        if not line:
            print "Note: unexpected end of file"
            break
        #print line.strip()
        match = re_codebuf.match(line)
        if match:
            percent = dumpfile.tell() / filesize
            if percent >= nextp:
                print >> sys.stderr, '%d%%...' % int(100*percent),
                nextp += 0.1
            #size = int(match.group(2))
            #data = dumpfile.read(size)
            #assert len(data) == size
            codebuf = CodeBuf(match.group(6), match.group(3), match.group(4),
                              int(match.group(5)), long(match.group(1), 16),
                              int(match.group(2)))
            codebuf.dumpfile = dumpfile
            codebuf.vlocalsofs = buftable.pop()
            codebufs.append(codebuf)
        else:
            match = re_specdict.match(line)
            if match:
                addr = long(match.group(1), 16)
                if len(codeboundary) != cbsortedsize:
                    codeboundary.sort()
                    cbsortedsize = len(codeboundary)
                codebuf = codeat(addr-4)
                if codebuf is None:
                    raise "spec_dict with no matching code buffer", line
                while 1:
                    line = dumpfile.readline()
                    if len(line)<=1:
                        break
                    match = re_spec1.match(line)
                    assert match
                    codebuf.spec_dict(match.group(2), long(match.group(1), 16))
            elif re_vinfo_array.match(line):
                assert len(codebufs) == bufcount
                break
            else:
                match = re_bigbuffer.match(line)
                if match:
                    BigBuffer(dumpfile, long(match.group(1), 16),
                              int(match.group(2)))
                else:
                    raise "invalid line", line
    print >> sys.stderr, 'sorting...',
    if len(codeboundary) != cbsortedsize:
        codeboundary.sort()
    codemap = {}
    #cblist = []
    codebufs.reverse()
    for codebuf in codebufs:
        codebuf.complete_list = codebufs
        codebuf.codemap = codemap
        codemap.setdefault(codebuf.addr, []).insert(0, codebuf)
        #prev, next, bigbuf = codebuf.getboundary()
        #cblist.append((bigbuf.priority, codebuf.addr, codebuf))
    #cblist.sort()
    #codebufs[:] = [codebuf for priority, addr, codebuf in cblist]
    print >> sys.stderr, '100%'
    return codebufs

if __name__ == '__main__':
    if len(sys.argv) > 1:
        codebufs = readdump(sys.argv[1])
    else:
        codebufs = readdump()
    for codebuf in codebufs:
        print codebuf.disassemble()



import os
import psyco

def time(fn, *args):
    "Measure the execution time of fn(*args)."
    import time, traceback
    begin = time.clock()
    try:
        result  = fn(*args)
    except:
        end = time.clock()
        traceback.print_exc()
        result = '<exception>'
    else:
        end = time.clock()
    return result, end-begin


ZERO = 0

def f1(n):
    "Arbitrary test function."
    i = 0
    x = 1
    while i<n:
        j = ZERO
        while j<=i:
            j = j + 1
            x = x + (i&j)
        i = i + 1
    return x

def f2(x):
    "Trivial function."
    return x*2

def f3(n):
    "Prints f2(x) on all non-null items of the sequence n."
    for x in n:
        if x == 0:
            continue
        print f2(x)

def f4(filelist):
    "Count all characters of all files whose names are in filelist."
    result = [0]*256
    for filename in filelist:
        if filename[:4] == 'BIN:': 
            f = open(filename[4:], 'rb')
        else:
            f = open(filename, 'r')
        for line in f:
            for c in line:
                k = ord(c)
                result[k] += 1
        f.close()
    d = {}
    for i in range(256):
        if result[i]:
            d[chr(i)] = result[i]
    return d

def f5(filelist):
    "Same as f4() but completely loads the files in memory."
    result = [0]*256
    for filename in filelist:
        if filename[:4] == 'BIN:': 
            f = open(filename[4:], 'rb')
        else:
            f = open(filename, 'r')
        for c in f.read():
            k = ord(c)
            result[k] += 1
        f.close()
    d = {}
    for i in range(256):
        if result[i]:
            d[chr(i)] = result[i]
    return d

def f6(n, p):
    "Computes the factorial of n modulo p."
    factorial = 1
    for i in range(2, n+1):
        factorial = (factorial * i) % p
    return factorial

def mandelbrot(c):
    z = 0j
    for i in xrange(1000):
        z = z*z + c
        if abs(z) > 2:
            if i > 126-33: i = 126-33
            return chr(33+i)
    return " "

def f7(start, end, step):
    "Computes the Mandelbrot set. All args are complex numbers."
    width = int((end.real - start.real) / step.real)
    while start.imag < end.imag:
        line = [mandelbrot(start + n*step.real) for n in range(width)]
        print ''.join(line)
        start += complex(0, step.imag)

def mandelbrotbis(c_r, c_i):
    z_r = z_i = 0.0
    for i in xrange(1000):
        z_r, z_i = z_r*z_r-z_i*z_i + c_r, 2*z_r*z_i + c_i
        if z_r*z_r+z_i*z_i > 4.0:
            if i > 126-33: i = 126-33
            return chr(33+i)
    return " "

def f7bis((start_r, start_i), (end_r, end_i), (step_r, step_i)):
    "Computes the Mandelbrot set. All args are (real, imag) tuples."
    width = int((end_r - start_r) / step_r)
    while start_i < end_i:
        line = [mandelbrotbis(start_r + n*step_r, start_i) for n in range(width)]
        print ''.join(line)
        start_i += step_i

def f8():
    try:
        x = 1.0 / 0.0
    #except StandardError, e:
    #    print "in except clause:", e
    finally:
        print "in finally clause"
        x = 5
    print "x is now", x

def f9(n):
    for i in range(n):
        print i,
    print

def f10():
    apply(f9, (50,))

def f11(n):
    def subtracter(x, y=n):
        return x-y
    return subtracter(1000)


ALLTESTS = []

def go(f, *args):
    print '-'*78
    v1, t1 = time(psyco.proxy(f), *args)
    print v1
    print '^^^ computed by Psyco in %s seconds' % t1
    v2, t2 = time(f, *args)
    if v1 == v2:
        if t1 and t2:
            ok = '%.2f times faster' % (t2/float(t1))
            s = ', Psyco is %s' % ok
        else:
            ok = s = ''
        print 'Python got the same result in %s seconds%s' % (t2, s)
    else:
        print v2
        print '^^^ by Python in %s seconds' % t2
        print '*'*78
        print '!!! different results !!!'
        ok = 'DIFFERENT RESULTS'
    ALLTESTS.append((f.func_name,t2,t1,ok))

def print_results():
    import sys
    print
    print '='*23, 'Summary (Python %d.%d.%d)' % sys.version_info[:3], '='*23
    format = '%-15s  %-15s  %-15s  %s'
    print format % ('function', 'Python time', 'Psyco time', '')
    for r in ALLTESTS:
        print format % r

def go1(arg=2117):
    go(f1, arg)

FILEPATH = '../c'
FILELIST = [os.path.join(FILEPATH, s) for s in os.listdir(FILEPATH)]
FILELIST = [s for s in FILELIST if os.path.isfile(s) and s!='psyco.dump']

def go4(arg=FILELIST):
    go(f4, arg)

def go5(arg=FILELIST):
    go(f5, arg)

def go6(n=100000, p=100000000000001L):
    go(f6, n, p)

def go7(start=-2-1j, end=1+1j, step=0.04+0.08j):
    go(f7, start, end, step)

def go7bis(start=(-2.0, -1.0), end=(1.0, 1.0), step=(0.04, 0.08)):
    go(f7bis, start, end, step)


if __name__ == "__main__":
    go(f11, 50)
    go1()
    go4()
    go5()
    go6()
    go7()
    go7bis()
    go(f10)
    psyco.dumpcodebuf()
    print_results()



import sys, os, StringIO, psyco, psyco.logger

LOGFILE = 'log-regrtest-psyco'


#NO_SYS_GETFRAME = """using sys._getframe() fails with Psyco"""

#NO_THREAD = """XXX not reliable, check if Psyco is generally
#        unreliable with threads or if there is another problem"""

#NO_PICKLE = """pickles function objects that Psyco rebinds"""

#NO_SYS_EXC = """XXX Psyco does not set sys.exc_xxx upon exception"""


SKIP = {'test_gc': "test_gc.test_frame() does not create a cycle with Psyco's limited frames",
#        'test_thread': NO_THREAD,
#        'test_asynchat': NO_THREAD,
#        'test_extcall': 'prints to stdout a function object that Psyco rebinds',
        'test_descr': 'seems that it mutates user-defined types and Psyco does not like it at all',
#        'test_pickle': NO_PICKLE,
#        'test_cpickle': NO_PICKLE,
#        'test_re': NO_PICKLE,
#        'test_sre': NO_SYS_EXC,
#        'test_string': NO_SYS_EXC,
#        'test_unicode': NO_SYS_EXC,
#        'test_inspect': 'gets confused with Psyco rebinding functions',
        'test_profilehooks': 'profiling does not see all functions run by Psyco',
        'test_profile': 'profiling does not see all functions run by Psyco',
        'test_repr': 'self-nested tuples and lists not supported',
        'test_builtin': 'vars() and locals() not supported',
        'test_inspect': 'does not run even in Python (when called the way classicregrtest.py calls it) and leaves buggy temp files around',
        'test_trace': 'no line tracing with Psyco',
        'test_threaded_import': 'Python hang-ups',
        'test_hotshot': "PyEval_SetProfile(NULL,NULL) doesn't allow Psyco to take control back",
        'test_coercion': 'uses eval() with locals',
        'test_weakref': 'incompatible with early unused variable deletion',
        }
#    SKIP['test_operator'] = NO_SYS_EXC
#    SKIP['test_strop'] = NO_SYS_EXC
if sys.version_info[:2] >= (2,3):
    SKIP['test_threadedtempfile'] = 'Python bug: Python test just hangs up'

if hasattr(psyco._psyco, 'VERBOSE_LEVEL'):
    SKIP['test_popen2'] = 'gets confused by Psyco debugging output to stderr'

GROUP_TESTS = 5    # number of tests to run per Python process


if '' in sys.path:
    sys.path.remove('')  # don't import the test.py in this directory!

from test import regrtest, test_support

repeat_counter = 4


def alltests():
    import random
    # randomize the list of tests, but try to ensure that we start with
    # not-already-seen tests and only after go on with the rest
    try:
        os.unlink(LOGFILE)
    except OSError:
        pass
    filename = "tmp_tests_passed"
    try:
        f = open(filename)
        tests_passed = eval(f.read())
        f.close()
    except IOError:
        tests_passed = {}
    testlist = regrtest.findtests()
    testlist = [test for test in testlist if not tests_passed.has_key(test)]
    random.shuffle(testlist)
    testlist1 = tests_passed.keys()
    random.shuffle(testlist1)
    print '\t'.join(['Scheduled tests:']+testlist)
    if testlist1:
        print '%d more tests were already passed and are scheduled to run thereafter.' % len(testlist1)
    testlist += testlist1
    while testlist:
        print '='*40
        tests1 = testlist[:GROUP_TESTS]
        del testlist[:GROUP_TESTS]
        err = os.system('"%s" %s %s' % (sys.executable, sys.argv[0],
                                          ' '.join(tests1)))
        if err:
            print '*** exited with error code', err
            return err
        for test in tests1:
            tests_passed[test] = 1
        f = open(filename, 'w')
        f.write(repr(tests_passed))
        f.close()
    print "="*60
    print
    print "Classic Regression Tests with Psyco successfully completed."
    print "All tests that succeeded twice in the same Python process"
    print "also succeeded %d more times with Psyco activated." % repeat_counter
    print
    try:
        os.unlink(filename)
    except:
        pass
    print "Psyco compilation flags:",
    d = psyco._psyco.__dict__
    if not d.has_key('ALL_CHECKS'):
        print "Release mode",
    for key in d.keys():
        if key == key.upper() and type(d[key]) == type(0):
            print "%s=%s" % (key, hex(d[key])),
    print

def python_check(test):
    if SKIP.has_key(test):
        print '%s skipped -- %s' % (test, SKIP[test])
        return 0
    for i in range(min(repeat_counter, 2)):
        print '%s, Python iteration %d' % (test, i+1)
        ok = regrtest.runtest(test, 0, 0, 0)
        special_cleanup()
        if ok <= 0:
            return 0   # skipped or failed -- don't test with Psyco
    return 1

def main(testlist, verbose=0, use_resources=None):
    if use_resources is None:
        use_resources = []
    test_support.verbose = verbose      # Tell tests to be moderately quiet
    test_support.use_resources = use_resources
    
    if type(testlist) == type(""):
        testlist = [testlist]
    if not verbose:
        testlist = filter(python_check, testlist)

    # Psyco selective compilation is only activated here
    psyco.log(LOGFILE, 'a')
    for test in testlist:
        psyco.logger.write('with test ' + test, 1)
    psyco.full()
    #print "sleeping, time for a Ctrl-C !..."
    #import time; time.sleep(1.5)


    for test in testlist:
        for i in range(repeat_counter):
            print '%s, Psyco iteration %d' % (test, i+1)
            ok = regrtest.runtest(test, 0, verbose, 0)
            special_cleanup()
            if ok == 0:
                return 0
            elif ok < 0:
                break
    return 1

def special_cleanup():
    try:
        dircache = sys.modules['dircache']
    except KeyError:
        pass
    else:
        for key in dircache.cache.keys():
            del dircache.cache[key]


if __name__ == '__main__':
    if len(sys.argv) <= 1:
        sys.exit(alltests() or 0)
    else:
        try:
            err = not main(sys.argv[1:])
        finally:
            # Write psyco.dump
            psyco.dumpcodebuf()
        if err:
            sys.exit(2)



import psyco, dis, types, sys, math

def f(filename, mode='r'):
    s = ""
    for line in open(filename, mode).readlines():
        s += "! " + line  # extreme optimization case
    return s

def f2(x, y, z):
    coords = (x, y)
    if type(z) is types.IntType:
        coords += (z,)
    else:
        coords += z
    a, b, c = coords
    return c, b, a

def f3(x, y):
    return x << y, x << 1, x << 31, x >> y, x >> 1, x >> 31

def f4(s):
    return "abc" + s

def double2(x):
    return x * 2

def overflowtest():
    n = double2(-1925703681)
    return `n`

def booltest():
    True,False = 1==1,1==0
    print 'abcdefghijklmnop'=='abcdefghijklMNop'
    print 'abcdefghijklmnop'=='abcdefghijklmnop'
    print str('abcdefghijklmnop'=='abcdefghijklMNop')
    print str('abcdefghijklmnop'=='abcdefghijklmnop')
    print (lambda x: x==1)(0)
    print (lambda x: x==1)(1)
    print [a & b for a in (False,True) for b in (False,True)]
    print [a | b for a in (False,True) for b in (False,True)]
    print [a ^ b for a in (False,True) for b in (False,True)]
    print not (False in ())

def exc_test():
    try:
        failure = [][1]   # opcode 9, line 2 from the start of the function
    except:
        exc, value, tb = sys.exc_info()
        print exc.__name__, str(value)
        print tb.tb_lineno - tb.tb_frame.f_code.co_firstlineno

def seqrepeat():
    for i in [5, 6L, -999]:
        print `i*'abc'`
        print `'abc'*i`
        print i*[3,'z']
        print [6,3]*i
        print `('y'*i) + 'x'`

def inplacerepeat():
    l = l1 = range(100)
    l *= 2
    return l is l1

def g5(x):
    return x+1

def f5(n):
    "Function inlining tests"
    print g5(n)
    print g5(n)
    print g5(3)
    print g5(x=n)

def g6(x):
    if x != 3:
        return x+1
    else:
        return None

def g6bis(n):
    return range(2, 50)[n]

def f6(n):
    "Function inlining tests"
    try:
        print g6(n)
        print g6(n)
        print g6(3)
        print g6(x=n)
        print g6bis(n)
    except:
        print sys.exc_type.__name__

def g7(x, *args):
    if not g7len(args):
        last = g5(0)
    else:
        last = args[-1]
    return last * x

def g7bis(tuple):
    return 0.0

def f7(n):
    global g7len
    g7len = len
    "Function inlining tests"
    print [g7(n), 5*g7(3,4,5), g7(*((11,)*5))]
    g7len = g7bis
    print [g7(n), 5*g7(3,4,5), g7(*((11,)*5))]

def g8(x, *args):
    return g7len(args) * x

def f8(n):
    global g7len
    g7len = len
    "Function inlining tests"
    print [g8(n), 5*g8(3,4,5), g8(*((11,)*5))]
    g7len = g7bis
    print [g8(n), 5*g8(3,4,5), g8(*((11,)*5))]

def f9(n):
    return g5(3), g5(n)


# virtual string tests

TEST = ['userhru', 'uiadsfz', '', '1', 'if', '623', 'opad', 'oa8ua',
        '09q34rx093q', '\x00',
        'qw09exdqw0e9dqw9e8d8qw9r8qw', '',
        '\x1d\xd7\xae\xa2\x06\x10\x1a\x00a\xff\xf6\xee\x15\xa2\xea\x89',
        'akjsdfhqweirewru 3498cr 3849rx398du389du389dur398d31623']

def s1():
    s = ''
    for t in TEST:
        s += t
    return s

def s2():
    return "some" + "another"

def s3():
    return TEST[0] + '.'

def s4():
    return '.' + TEST[0]

def s5():
    return TEST[0] + (TEST[1] + TEST[2])

def s6():
    s = ''
    for t in TEST:
        s = t + s
    return s

def s7():
    return int(TEST[-1][5:10] == 'fhqwe')

def s8():
    return TEST[-2][5:10][3]

def s9():
    return TEST[-1][:-1][1:][2:-3]

def s10():
    return int('t' < TEST[1][:1] == 'u')

def s11():
    return int('ia' == TEST[1][1:3] < 'ib' and
               'dsf' == TEST[1][-4:-1] < 'dsf!' and
               'dsfz' == TEST[1][-4:] <= 'dsfz')

def s12():
    return int(TEST[0] + TEST[1] != 'userhruuiadsfz')

def s13():
    return int(TEST[3] + TEST[5] == TEST[-1][-4:])

def teststrings():
    print `s1()`
    print `s2()`
    print `s3()`
    print `s4()`
    print `s5()`
    print `s6()`
    print `s7()`
    print `s8()`
    print `s9()`
    print `s10()`
    print `s11()`
    print `s12()`
    print `s13()`

def testslices(s):
    "Test the various paths the code can take in pstring_slice()."
    print `s[:-9999]`
    print `s[:9999]`
    print `s[:]`
    print `s[-9999:-9999]`
    print `s[-9999:9999]`
    print `s[-9999:]`
    print `s[1:-9999]`
    print `s[1:9999]`
    print `s[1:]`
    print `s[9999:-9999]`
    print `s[9999:9999]`
    print `s[9999:]`

def testovf(x, y):
    print x+y
    print 2000000000+y
    print x+2000000000
    print x-y
    print 2000000000-y
    print x-2000000000
    print x*y
    print 2000000000*y
    print x*2000000000
    print x
    print -x
    print abs(x)

def rangetypetest(n):
    print type(range(n)).__name__
    print type(range(2, n)).__name__
    print type(range(n, 2, -1)).__name__
    print type(xrange(n)).__name__
    print type(xrange(2, n)).__name__
    print type(xrange(n, 2, -1)).__name__

def rangetest(n):
    for i in range(n):
        print i,
    print
    for i in range(10, n):
        print i,
    print
    for i in range(n, 10, -1):
        print i,
    print

def xrangetest(n):
    for i in xrange(n):
        print i,
    print
    for i in xrange(10, n):
        print i,
    print
    for i in xrange(n, 10, -1):
        print i,
    print

def longrangetest():
    L = 1234567890123456789L
    if sys.hexversion >= 0x02030000:
        print range(L, L+2)
    else:
        print [L, L+1]

def proxy_defargs():
    d = {}
    exec """
def f():
    g()
def g(a=12):
    print a
""" in d
    d['f'] = psyco.proxy(d['f'])
    d['g'] = psyco.proxy(d['g'])
    d['f']()

def setfilter():
    def f1():
        return ('f1', int(__in_psyco__))
    def f2():
        "TAGGY"
        return ('f2', int(__in_psyco__))
    def myfilter(co):
        return co.co_consts[0] != 'TAGGY'
    psyco.setfilter(myfilter)
    try:
        print psyco.proxy(f1)()  # ('f1', 1)
        print psyco.proxy(f2)()  # ('f2', 0)
    finally:
        prev = psyco.setfilter(None)
        assert prev is myfilter

if sys.version_info < (2,3):
    def arraytest():
        print "S"
else:
    def arraytest():
        import array
        class S(array.array): pass
        print type(S('i')).__name__


class CompositeElement:
    def __init__(self):
        self.children = []

    def append(self, child):
        self.children.append(child)
        return self

def makeSelection():
    """Segfault bug (thanks xorAxAx)"""
    result = CompositeElement()
    result.append(CompositeElement().append(42))
    print 'do stuff here'
    assert isinstance(result.children[0], CompositeElement)

class Class1:
    __metaclass__ = type
def class_creation_1(n=400000):
    for i in xrange(n):
        Class1()
    print "ok"

class Class2:
    __metaclass__ = type
    def __init__(self, a):
        self.a = a
def class_creation_2(n=400000):
    for i in xrange(n):
        c = Class2(i)
        assert c.a == i
    print "ok"

class Class3:
    __metaclass__ = type
    def __init__(self):
        return 42
def class_creation_3():
    try:
        Class3()
    except TypeError:
        if (2,2) <= sys.version_info < (2,5):
            print "got a TypeError, but new-style classes don't"
            print "check the __init__() return value before 2.5"
        else:
            print "ok"
    else:
        if (2,2) <= sys.version_info < (2,5):
            print "ok"
        else:
            print "__init__() => 42: should have raised TypeError!"

class Class4:
    __metaclass__ = type
    def __init__(self, a=None):
        self.a = a
def class_creation_4(n=200000):
    for i in xrange(n):
        c = Class4(a=i)
        assert c.a == i
    print "ok"

def power_int(n=2000):
    for j in range(n):
        x = 0
        for i in range(1000):
            x += i ** 2
    print x

def power_int_long(n=200):
    for j in range(n):
        x = 0
        for i in range(1000):
            x += i ** 2L
    print type(x).__name__[:4], int(x)

def power_float(n=500):
    for j in range(n):
        x = 0.0
        for i in range(1000):
            x += i ** 1.409999992
    print type(x).__name__, int(x)

def conditional_doubletest_fold():
    n = sys.getrefcount(conditional_doubletest_fold)   # anything > 0
    if n < 0 and expression_not_evaluated:
        print "BUG"
    else:
        print "ok(1)"
    n = sys.getrefcount(conditional_doubletest_fold)   # anything > 0
    if n >= 0 or expression_not_evaluated:
        print "ok(2)"
    else:
        print "BUG"

def importname(value):
    for v in value:
        import sys
    print "Ok"

def sharedlists(n):
    if n > 5:
        a = b = [4, 5]
    else:
        a = [4, 5]
        b = [4, 5]
    double2(21)
    a[0] = 8
    print b[0]

def variousslices():
    class X(object):
        def __getitem__(self, x):
            print x
    class Y(object):
        def __getitem__(self, x):
            print x
        def __getslice__(self, i, j):
            print i, j
    for o in [X(), Y()]:
        o[4:]
        o[:7]
        o[9:'hello']
        o['world':]
        o[1:10:'hello']

def listgetitem():
    class X(list):
        def __getitem__(self, key):
            return key
    x = X(["Ok"])
    print x["foobar"]
    for item in x:
        print item

def negintpow(x):
    print x ** -2

if __name__ == '__main__':
    from test1 import go, print_results
    import time
    print "break!"
    time.sleep(0.5)
    #go(class_creation_1)
    #go(class_creation_2)
    #go(class_creation_3)
    #go(class_creation_4)
    #go(power_int)
    #go(power_int_long)
    #go(power_float)
    psyco.proxy(negintpow)(8)
    psyco.dumpcodebuf()
    print_results()



import re, dis
recase = re.compile(r'case\s+([A-Z_][A-Z_+0-9]*)\s*[:]')
rebin  = re.compile(r'BINARY_OPCODE[(]([A-Z_][A-Z_+0-9]*),')
remiss = re.compile(r'[/*]*MISSING_OPCODE[(]([A-Z_][A-Z_+0-9]*)[)]')

if dis.opname[86] == 'YIELD_STMT':
    dis.opname[86] = 'YIELD_VALUE'

inswitch = 0
lst = []
miss = []
warn = 0

def register(op):
    global warn
    print "%29s" % op,
    if op not in lst:
        lst.append(op)
    if op not in dis.opname:
        print "%29s" % "<--- unknown opcode",
        #warn += 1
    print

print "+++ Found +++"
print

for line in open('Python/pycompiler.c').readlines():
    line = line.strip()
    if line.startswith('switch (opcode)'):
        inswitch = 1
    elif line.endswith('/* switch (opcode) */'):
        inswitch = 0
    elif inswitch:
        m = recase.match(line) or rebin.match(line)
        if m:
            register(m.group(1))
        else:
            m = remiss.match(line)
            if m:
                miss.append(m.group(1))
print
print

assert not inswitch

print '+++ Not implemented +++'
print
for i, opname in zip(range(1,256), dis.opname[1:]):
    if not opname.startswith('<') and opname not in lst:
        print "%4d %29s" % (i, opname),
        if opname not in miss:
            print "%29s" % "<--- forgotten ?",
            warn += 1
        else:
            miss.remove(opname)
        print
print

for opname in miss:
    print "%29s" % opname, "%29s" % "<--- marked as missing, don't know why"

print

for line in open('mergepoints.c').readlines():
    for word in lst:
        if line.find(word) >= 0:
            lst.remove(word)
            break

for word in lst:
    print '!!! Not found in mergepoints.c !!!'
    print
    print word
    print
    warn += 1

if warn:
    print '                               !!! %d warning(s) !!!' % warn



#
# Search for identical lines in stdin. Line starting with "<number>*"
# are assumed to be repeated that number of times.
# stdout report is formatted according to sys.argv[1].
#
import sys, re

re1 = re.compile(r"([0-9]+)[*](.*)")

def samelines(infile, outfile, format, verbose=1, minimum=2):
    lines = {}
    total = 0
    try:
        for line in infile:
            verbose -= 1
            if not verbose:
                if total:
                    print >> sys.stderr, '%d lines, %d without duplicates...' % (
                        total, len(lines))
                total += 5000
                verbose = 5000
            if line.endswith('\n'):
                line = line[:-1]
            match = re1.match(line)
            if match:
                count = int(match.group(1))
                line = match.group(2)
            else:
                count = 1
            lines[line] = lines.get(line, 0) + count
    finally:
        for line, count in lines.iteritems():
            if count >= minimum:
                print >> outfile, format % (line, count)

if __name__ == '__main__':
    format = sys.argv[1]
    samelines(sys.stdin, sys.stdout, format)



from insnset import InstructionSet


 --- IN-PROGRESS ---


class Immed:
    def __init__(self, *commonvalues):
        self.commonvalues = commonvalues

    def modes(self, ivm):
        for n in self.commonvalues:
            if ivm.nextarg('void', min=n, max=n) is not None:
                return str(n)
        return ivm.nextarg('word_t')

class Address:
    def modes(self, ivm):
        return ivm.nextarg('word_t')

class Stack:
    def modes(self, ivm):
        for i in range(len(ivm.stack)):
            if ivm.nextarg('void', min=i, max=i) is not None:
                return ivm.stack[i]
        n = ivm.nextarg('code_t', min=len(stack), max=255)
        if n is None:
            n = ivm.nextarg('word_t')
        return ivm.compute('stack_nth(%s-%d)' % (n, len(stack)))

class StackRef:
    def modes(self, ivm):
        ...


class IVM(InstructionSet):

    def insn_inv(self):
        x = self.pop(); self.push('~%s' % x)

    def insn_neg_o(self):
        x = self.pop(); self.push('-%s' % x)
        self.setflag('%s == LONG_MIN' % x)

    def insn_abs_o(self):
        x = self.pop(); self.push('%s<0 ? -%s : %s' % (x,x,x))
        self.setflag('%s == LONG_MIN' % x)

    def insn_add(self):
        y = self.pop(); x = self.pop(); self.push('%s+%s' % (x,y))

    def insn_add_o(self):
        y = self.pop(); x = self.pop(); self.push('%s+%s' % (x,y))
        self.setflag('((%s+%s)^%s) < 0 && (%s^%s) >= 0' % (x,y,x,x,y))

    def insn_immed(self, x=Immed(0,1)):
        self.push(x)

    def insn_s_push(self, s=Stack()):
        self.push(s)

    def insn_s_pop(self, s=StackRef()):
        s.write(self.pop())

    def insn_flag_push(self):
        self.push(self.consumeflag())

    def insn_cmpz(self):
        self.pop('x'); self.setflag('x == 0')

    def insn_jumpfar(self, target=Address()):
        self.do('nextip = (code_t*) %s;' % target)
    insn_jumpfar.chainable = False

    def insn_load1(self):
        self.pop('addr'); self.push('*(char*) addr')

    def insn_store1(self):
        self.pop('value'); self.pop('addr')
        self.do('*(char*) addr = value;')


ivm = IVM(['accum'])
ivm.insn_add_o()
ivm.insn_add()
ivm.insn_flag_push()
ivm.normalize(['accum'])
ivm.write()



import sys, os
import xam

"""
This script loads a psyco.dump file (like httpxam.py) and
reads on its standard input a list of addresses. For each
address that it recognizes it prints the name of the
corresponding symbol or the address of the code buffer
that contains the address. Use this on debugger memory
dumps.

This could be enhanced by detecting the addresses of
vinfo_t's as well.
"""


def main(codebufs, f):
    while 1:
        line = f.readline()
        if not line:
            break
        for addr in xam.lineaddresses(line):
            sym = xam.symbols.get(addr)
            if sym:
                print '0x%x\tis\t' % addr, xam.symtext(sym, addr)
                break


if __name__ == '__main__':
    if len(sys.argv) <= 1:
        print "Usage: python symbols.py <directory>"
        print "  psyco.dump is loaded from the <directory>."
        sys.exit(1)
    DIRECTORY = sys.argv[1]
    del sys.argv[1]
    codebufs = xam.readdump(os.path.join(DIRECTORY, 'psyco.dump'))
    print >> sys.stderr, "Reading for addresses from stdin..."
    main(codebufs, sys.stdin)



#
# A less memory-exploding version of regrtester.py.
# It runs only a fraction of the tests.
#

import sys, re, psyco


assert len(sys.argv) >= 2
match = re.match(r"(\d+)[/](\d+)", sys.argv[1])
assert match, "syntax: regrtester2.py n/m [-nodump] [seed]"
n = int(match.group(1))
m = int(match.group(2))
assert 0 <= n < m


import test.regrtest
import regrtester

def confirm_still_in_psyco():
    return __in_psyco__

tests = [s for s in test.regrtest.findtests()
         if hash(s) % m == n and s not in test.regrtest.NOTTESTS]
if __name__ == '__main__':
    if len(sys.argv) > 2 and sys.argv[2] == '-nodump':
        dump = 0
        del sys.argv[2]
    else:
        dump = 1
        
    import random, time
    seed = time.ctime()
    if len(sys.argv) > 2:
        seed = sys.argv[2]
        del sys.argv[2]
    print 'Random seed is %r' % seed
    random.seed(seed)
    random.shuffle(tests)

    fully_in_psyco = confirm_still_in_psyco()
    try:
        test.regrtest.main(tests)  #, verbose=1)
    finally:
        if dump:
            psyco.dumpcodebuf()
        if fully_in_psyco:
            assert confirm_still_in_psyco()



import os, sys, files
from mkincl import *

GCC = '/usr/bin/gcc'

processor = sys.argv[1]


g = open('dyn-Makefile', 'w')
print >> g, "# automatically generated by depend.py"
print >> g
print >> g, "O_FILES =",

basenames = []
for f in files.SRC:
    assert f.filename.endswith('.c')
    basenames.append(f.filename[:-2])
for f in files.PROCESSOR_SRC[processor]:
    assert f.filename.endswith('.c')
    basenames.append(processor + '/' + f.filename[:-2])

for fn in basenames:
    print >> g, fn+'.o',
print >> g
print >> g

for fn in basenames:
    print fn
    h = os.popen("%s -I%s %s -M -Wundef %s.c" % (GCC, processor,
                                                  INCLUDE_STR, fn), 'r')
    data = h.read()
    h.close()
    data = data.replace('\\\n', ' ')
    data = data[data.index(':')+1:]
    data = data.strip().split()
    data = map(os.path.normpath, data)
    found = {}
    print >> g, '%s.o:' % fn,
    for f1 in data:
        if f1 not in found and not f1.startswith('/'):
            print >> g, f1,
            found[f1] = 1
    print >> g
    print >> g



#! /usr/bin/env python
#
# The Ultimate Test Suite (tm)
#
import sys, os, time

#
# This script should do the following:
#
#  for interpreter-executable-path in file('fulltester.local'):
#      for mode in (debug, optimized, ivm-debug, ivm-optimized):
#          compile Psyco for 'mode' and 'interpreter'
#          for testset in testset_list:
#              start 'interpreter' running 'testset'
#
# The testset_list is called RUNNING_MODES below, and has the 
# following entries:

# test_base.py         -- run elementary tests
# regrtester2.py 0/10  -- 1st tenth of all regr tests with psyco.full()
# regrtester2.py 1/10  -- 2nd tenth of all regr tests with psyco.full()
# ...
# regrtester2.py 9/10  -- last tenth of all regr tests with psyco.full()
# (idem)              -- the same, with psyco.profile()
#

try:
    f = open('fulltester.local')
except IOError:
    print >> sys.stderr, "You must create a file 'fulltester.local' containing a"
    print >> sys.stderr, "list of Python executables of various versions."
    sys.exit(2)

PYTHON_VERSIONS = [s.strip() for s in f.readlines()]
f.close()

PSYCO_MODES = [
    # debugging mode, static compiling
    {'PSYCO_DEBUG': 1, 'VERBOSE_LEVEL': 1, 'CODE_DUMP': 1, 'ALL_STATIC': 1},
    # optimized mode, static compiling
    {'PSYCO_DEBUG': 0, 'VERBOSE_LEVEL': 0, 'CODE_DUMP': 0, 'ALL_STATIC': 1},
    # debugging mode, IVM virtual processor
    {'PSYCO_DEBUG': 1, 'VERBOSE_LEVEL': 1, 'CODE_DUMP': 1, 'ALL_STATIC': 1,
     'PROCESSOR': 'ivm'},
    # optimized mode, IVM virtual processor
    {'PSYCO_DEBUG': 0, 'VERBOSE_LEVEL': 0, 'CODE_DUMP': 0, 'ALL_STATIC': 1,
     'PROCESSOR': 'ivm'},
    ]

FRACTION = 10

RUNNING_MODES = [
        ("test_base.py",  {}),
    ]
for n in range(FRACTION):
    RUNNING_MODES.append(
        ("regrtester2.py %d/%d -nodump" % (n, FRACTION),
                          {"regrtester.local": "psyco.full()"}))
for n in range(FRACTION):
    RUNNING_MODES.append(
        ("regrtester2.py %d/%d -nodump" % (n, FRACTION),
                          {"regrtester.local": "psyco.profile()"}))

compiled_version = None


def run(path, *argv):
    print '='*10, path, ' '.join(argv)
    sys.stdout.flush()
    path = os.path.expanduser(path)
    err = os.spawnv(os.P_WAIT, path, [path]+list(argv))
    if err:
        print '='*60
        print '*** exited with error code', err
        sys.exit(err)

def do_compile(python_version, psyco_mode):
    print '~'*60
    print psyco_mode
    preffile = os.path.join(os.pardir, 'preferences.py')
    try:
        f = open(preffile, 'r')
        backup = f.read()
        f.close()
    except IOError:
        backup = ''
    if backup:
        print "===== Note: ignoring your custom preferences.py"
    cwd = os.getcwd()
    try:
        f = open(preffile, 'w')
        for varvalue in psyco_mode.items():
            f.write('%s = %r\n' % varvalue)
        f.close()
        os.chdir(os.pardir)
        run(python_version, 'setup.py', 'build', '-f')
        run(python_version, 'setup.py', 'install_lib')
    finally:
        os.chdir(cwd)
        f = open(preffile, 'w')
        f.write(backup)
        f.close()

def test_with(python_version, psyco_mode, running_mode):
    #
    # Compile the appropriate Psyco version
    #
    global compiled_version
    if (python_version, psyco_mode) != compiled_version:
        do_compile(python_version, psyco_mode)
        compiled_version = (python_version, psyco_mode)
    #
    # Prepare the running mode
    #
    script, prefiles = running_mode
    for filename, content in prefiles.items():
        f = open(filename, 'w')
        print >> f, content
        f.close()
    #
    # Run it
    #
    print '='*10, '%s: %s' % (
        psyco_mode['PSYCO_DEBUG'] and "DEBUG" or "RELEASE", 
        running_mode)
    run(python_version, *script.split())
    #
    # Passed
    #
    tests_passed.append((python_version, psyco_mode, running_mode))
    f = open(passed_filename, 'w')
    f.write(repr(tests_passed))
    f.close()
    #
    # Make a little pause to catch our breath again
    #
    time.sleep(1)


def test():
    global tests_passed, passed_filename
    passed_filename = "tmp_fulltests_passed"
    try:
        f = open(passed_filename)
        tests_passed = eval(f.read())
        f.close()
    except IOError:
        tests_passed = []

    tests = [(python_version, psyco_mode, running_mode)
             for python_version in PYTHON_VERSIONS
             for psyco_mode in PSYCO_MODES
             for running_mode in RUNNING_MODES]
    tests.sort()
    tests.reverse()
    later_tests = []
    for mode in tests:
        if mode in tests_passed:
            later_tests.append(mode)
        else:
            test_with(*mode)
    for mode in later_tests:
        test_with(*mode)
    try:
        os.unlink(passed_filename)
    except:
        pass
    try:
        os.unlink("regrtester.local")
    except:
        pass
    print "="*60
    print
    print "The Ultimate Test Suite (tm) passed !"
    print
    print "="*60


if __name__ == '__main__':
    test()





class InstructionSet:

    def __init__(self, stack):
        self.locals = {}
        self.stack = stack
        self.popped = 0
        self.flag = 'flag'
        self.codelines = []

    def do(self, code):
        self.codelines.append(code)

    def push(self, expr):
        self.stack.append(expr)

    def compute(self, expr):
        if expr in self.locals:
            return expr
        name = 'l%d' % len(self.locals)
        self.locals[name] = expr
        self.do('%s = %s;' % (name, expr))
        return name

    def pop(self):
        if self.stack:
            name = self.compute(self.stack.pop())
        else:
            name = self.compute('stack_nth(%d);' % self.popped)
            self.popped += 1
        return name

    def setflag(self, expr):
        self.flag = expr

    def forgetflag(self):
        self.flag = None

    def consumeflag(self):
        result = self.flag
        self.forgetflag()
        return result

    def normalize(self, stack):
        rstack = list(stack)
        rstack.reverse()
        code = ['%s = %s;' % (target, self.pop()) for target in rstack]
        shift = self.popped-len(self.stack)
        code += ['stack_nth(%d) = %s;' % (i, self.pop())
                 for i in range(len(self.stack))]
        if self.flag is not None and self.flag != 'flag':
            self.do('flag = %s;' % self.flag)
        if shift != 0:
            if shift > 0:
                self.do('stack_shift_pos(%d);' % shift)
            else:
                self.do('stack_shift(%d);' % shift)
        for line in code:
            self.do(line)
        self.stack = list(stack)
        self.popped = 0

    def write(self):
        print '{'
        if self.locals:
            locals = self.locals.keys()
            locals.sort()
            print '\tword_t %s;' % (', '.join(locals),)
        for line in self.codelines:
            print '\t' + line
        print '}'


class InstructionSetWriter:

    def __init__(self, InsnSet, nbaccum=1):
        self.InsnSet = InsnSet
        self.nbaccum = nbaccum

    



"""
python ivmoptimize.py path [ path ... ]

This script optimizes and regenerates the 'ivm' virtual machine
used by Psyco on non-i386 platforms.  You need to compile Psyco
in debugging mode (see below) and use 'psyco.dumpcodebuf()' to
generate one or more dump files called 'psyco.dump'.  Run then
the present script with the path(s) to the 'psyco.dump' file(s).
Finally, you have to recompile Psyco in normal (optimized) mode.

You need SWI Prolog to do that.  http://www.swi-prolog.org/

To compile Psyco in debugging mode, create a file 'preferences.py'
in the same directory as 'setup.py' with the following content:

PROCESSOR = 'ivm'
PSYCO_DEBUG = 1
VERBOSE_LEVEL = 1
CODE_DUMP = 1

and re-run 'python setup.py build -f install'.
"""
import sys, os
import ivmextract

try:
    LOCALDIR = __file__
except NameError:
    LOCALDIR = sys.argv[0]
LOCALDIR = os.path.dirname(LOCALDIR)


def main(paths, maxlength=8, optmode='optimize.pl'):
    outfilenames = [os.path.abspath(ivmextract.main(dir)) for dir in paths]
    os.chdir(os.path.join(LOCALDIR, os.pardir, 'c', 'ivm', 'prolog'))
    g = open("mode_combine.pl", "w")
    g.close()      # empty file
    g = os.popen('pl -f %s -g remotecontrol -t halt' % optmode, 'w')
    for fn in outfilenames:
        print >> g, "loaddumpfile('%s')." % fn
    print >> g, "measure(%d)." % maxlength
    print >> g, "emitmodes(255)."
    g.close()
    g = open("mode_combine.pl", "r")
    if not g.readline():
        print >> sys.stderr, "*** the Prolog program %s failed" % optmode
        sys.exit(1)
    g.close()
    err = os.system('pl -f insns.pl -g main_emit -t halt')
    if err == 0:
        print
        print 'Done.  If you compile Psyco, its ivm virtual machine will now'
        print 'be optimized for the usage patterns found in the dump files.'


if __name__ == '__main__':
    if len(sys.argv) <= 1:
        print >> sys.stderr, __doc__
        sys.exit(2)
    else:
        main(sys.argv[1:])



import sys, os

def data2unix(lines):
    data = []
    modif = 0
    for line in lines:
        line1 = line.replace('\r', '')
        modif = modif + (line!=line1)
        data.append(line1)
    return data, modif

def data2win(lines):
    data = []
    modif = 0
    for line in lines:
        if line[-1:] == '\n':
            line1 = line[:-1].replace('\r', '') + '\r\n'
            modif = modif + (line!=line1)
        data.append(line1)
    return data, modif

def file2unix(name, verbose=1):
    "Turn a file's end-of-line into Unix flavor without changing the timestamp."
    s = os.stat(name)
    f = open(name, 'rb')
    data, modif = data2unix(f)
    f.close()
    if modif:
        if verbose:
            print "Unix'ed %d lines in %s" % (modif, name)
        f = open(name, 'wb')
        f.writelines(data)
        f.close()
    os.utime(name, (s.st_atime, s.st_mtime))
    return modif

def file2win(name, verbose=1):
    "Turn a file's end-of-line into Windows flavor without changing the timestamp."
    s = os.stat(name)
    f = open(name, 'rb')
    data, modif = data2win(f)
    f.close()
    if modif:
        if verbose:
            print "Win'ed %d lines in %s" % (modif, name)
        f = open(name, 'wb')
        f.writelines(data)
        f.close()
    os.utime(name, (s.st_atime, s.st_mtime))
    return modif

class Directory:
    def __init__(self, srcpath, relpath=''):
        self.path = srcpath
        self.relpath = relpath
        self.subdirs = []
        self.fileinfo = {}
        entryname = os.path.join(srcpath, 'CVS', 'Entries')
        try:
            f = open(entryname, 'r')
        except IOError:
            print >> sys.stderr, "note: cannot read", entryname
            return
        lines = f.readlines()
        f.close()
        entryname2 = os.path.join(srcpath, 'CVS', 'Entries.Log')
        try:
            f = open(entryname2, 'r')
        except IOError:
            pass
        else:
            for line in f.readlines():
                if line[:2] == 'A ':
                    lines.append(line[2:])
                elif line[:2] == 'R ':
                    lines.remove(line[2:])
            f.close()
        for line in lines:
            line = line.split('/')
            if len(line) >= 6:
                fname1 = line[1]
                if 'D' in line[0]:
                    self.subdirs.append(fname1)
                else:
                    self.fileinfo[fname1] = line
    def subdir(self, name):
        return Directory(os.path.join(self.path, name),
                         os.path.join(self.relpath, name))
    def alldirs(self):
        result = [self]
        for name in self.subdirs:
            result += self.subdir(name).alldirs()
        return result
    def unknownfiles(self):
        try:
            return [filename for filename in os.listdir(self.path)
                    if not self.fileinfo.has_key(filename) and
                       filename not in self.subdirs and
                       filename != 'CVS']
        except OSError:
            return []


if __name__ == '__main__':
    # print the full name of all the files
    root = Directory('.')
    for dir in root.alldirs():
        for name, info in dir.fileinfo.items():
            print os.path.join(dir.path, name)

##    # Example: print the full name of the files with revision 1.1.1.1
##    root = Directory('.')
##    for dir in root.alldirs():
##        for name, info in dir.fileinfo.items():
##            if info[2] == '1.1.1.1':
##                print os.path.join(dir.path, name)



#from __future__ import generators
#from __future__ import division
import psyco, os, sys, array, operator, traceback
from time import sleep, clock

def time(fn, *args):
    "Measure the execution time of fn(*args)."
    begin = clock()
    try:
        result  = apply(fn, args)
    except:
        end = clock()
        traceback.print_exc()
        result = '<exception>'
    else:
        end = clock()
    return result, end-begin


ZERO = 0

def f1(n):
    "Arbitrary test function."
    i = 0
    x = 1
    while i<n:
        j = ZERO
        while j<=i:
            j = j + 1
            x = x + (i&j)
        i = i + 1
    return x

def f2(x):
    "Trivial function."
    return x*2

def f3(n):
    "Prints f2(x) on all non-null items of the sequence n."
    for x in n:
        if x == 0:
            continue
        print f2(x)

def f4(filelist):
    "Count all characters of all files whose names are in filelist."
    result = [0]*256
    for filename in filelist:
        f = open(filename, 'r')
        for line in f:
            for c in line:
                k = ord(c)
                result[k] = result[k] + 1
        f.close()
    d = {}
    for i in range(256):
        if result[i]:
            d[chr(i)] = result[i]
    return d

def f5(filelist):
    "Same as f4() but completely loads the files in memory."
    result = [0]*256
    for filename in filelist:
        f = open(filename, 'r')
        for c in f.read():
            k = ord(c)
            result[k] = result[k] + 1
        f.close()
    d = {}
    for i in range(256):
        if result[i]:
            d[chr(i)] = result[i]
    return d

def f6(n, p):
    "Computes the factorial of n modulo p."
    factorial = 1
    for i in range(2, n+1):
        factorial = (factorial * i) % p
    return factorial

def mandelbrot(c):
    za, zb = c.real, c.imag
    for i in range(1000):
        za2 = za*za
        zb2 = zb*zb
        if za2+zb2 > 4.0:
            if i > 126-33: i = 126-33
            return chr(33+i)
        za, zb = za2-zb2+c.real, 2*za*zb+c.imag
        del za2, zb2
    return " "

def f7(start, end, step):
    "Computes the Mandelbrot set. All args are complex numbers."
    width = int((end.real - start.real) / step.real)
    while start.imag < end.imag:
        line = [mandelbrot(start + n*step.real) for n in range(width)]
        print ''.join(line)
        start += complex(0, step.imag)

def f8():
    try:
        x = 1.0 / 0.0
    #except StandardError, e:
    #    print "in except clause:", e
    finally:
        print "in finally clause"
        x = 5
    print "x is now", x

def f9(n):
    for i in range(n):
        print i,
    print

def f10():
    apply(f9, (50,))


def go(f, *args):
    print '-'*80
    v1, t1 = time(psyco.proxy(f), *args)
    print v1
    print '^^^ computed by Psyco in %s seconds' % t1
    v2, t2 = time(f, *args)
    if v1 == v2:
        if t1 and t2:
            s = ', Psyco is %.2f times faster' % (t2/float(t1))
        else:
            s = ''
        print 'Python got the same result in %s seconds%s' % (t2, s)
    else:
        print v2
        print '^^^ by Python in %s seconds' % t2
        print '*'*80
        print '!!! different results !!!'

def go1(arg=2117):
    go(f1, arg)

FILEPATH = '../c'
FILELIST = [os.path.join(FILEPATH, s) for s in os.listdir(FILEPATH)]
FILELIST = [s for s in FILELIST if os.path.isfile(s) and s!='psyco.dump']

def go4(arg=FILELIST):
    go(f4, arg)

def go5(arg=FILELIST):
    go(f5, arg)

def go6(n=100000, p=100000000000001L):
    go(f6, n, p)

def go7(start=-2-1j, end=1+1j, step=0.04+0.08j):
    go(f7, start, end, step)


def f11(prefix):
    #prefix = [60, 115, 116, 114, 114, 114, 114, 62]
    
    table = [-1] + ([0]*len(prefix))
    for i in range(len(prefix)):
        table[i+1] = table[i]+1
        while table[i+1] > 0 and prefix[i] != prefix[table[i+1]-1]:
            table[i+1] = table[table[i+1]-1]+1
    #code.extend(table[1:]) # don't store first entry
    print table[1:]

def f12(arg):
    def g(x, y=arg):
        return x+y
    return g(123)

def f13(arg):
    x,y = arg
    print x, y
    u,v = x,y
    print u, v
    return v,u

def f15(x):
    print x

def f14(n):
    f15(n==5)

def f16(x):
    return ~x

def f17(x, y=0, *args):
    return x, y, args

#def f18():
#    yield 5
#    yield 6

def f19(seq):
    class C:
        def __init__(self):
            print "hello"
    c = C()
    seq = list(seq)
    seq1 = seq
    seq *= 3
    n = seq[0]
    n /= 3
    return seq, seq1 is seq, n

def f20(code, n, fill=0):
    a = array.array(code, [fill]*n)
    for i in range(n):
        item = i*10
        if code == 'c':
            item = chr(item)
        a[i] = item
    return [a[i] for i in range(len(a))]

def f21(lst):
    return operator.indexOf(lst, 9)

def f22(n):
    try:
        x = 1/n
    except:
        print "Catch!"

def f23(n):
    f22(n)
    try:
        x = undefined_name
    except:
        f22(n)
        print sys.exc_type.__name__

def f24(n):
    return 1/n

def f25(n):
    try:
        return f24(n)
    except AttributeError:
        auwsziazsi

def f26():
    import psyco.classes
    import array
    import urllib
    from types import FloatType, ListType
    from psyco.core import full
    return FloatType, ListType, urllib, psyco.classes, full

def test_getframe():
    import sys
    i = 0
    print 'test_getframe():'
    while 1:
        try:
            f = sys._getframe(i)
        except ValueError:
            break
        #print '%-26s %-60s %-40s' % (f, f.f_code, f.f_locals.keys())
        print f.f_code.co_name.replace('<module>', '?')
        i += 1

def test_getframe1():
    return test_getframe()
psyco.cannotcompile(test_getframe1)

def test_getframe_b():
    import sys
    i = 0
    print 'test_getframe_b():'
    f = sys._getframe()
    while f is not None:
        print f.f_code.co_name.replace('<module>', '?')
        f = f.f_back    # walk the stack with f_back

def test_getframe_b1():
    return test_getframe_b()
psyco.cannotcompile(test_getframe_b1)

def f28():
    test_getframe()
    return N

def f27():
    global N
    N = 5
    a = f28()
    N = 6
    b = f28()
    N = 7
    c = f28()
    return a,b,c

def f28_1():
    test_getframe1()
    return N

def f27_1():
    global N
    N = 51
    a = f28_1()
    N = 61
    b = f28_1()
    N = 71
    c = f28_1()
    return a,b,c

def f28_b():
    test_getframe_b()
    return N

def f27_b():
    global N
    N = 95
    a = f28_b()
    N = 96
    b = f28_b()
    N = 97
    c = f28_b()
    return a,b,c

def f28_b1():
    test_getframe_b1()
    return N

def f27_b1():
    global N
    N = 951
    a = f28_b1()
    N = 961
    b = f28_b1()
    N = 971
    c = f28_b1()
    return a,b,c

def f29(lst):
    lst = lst[:]
    del lst[2]
    lst[3] = 6
    return lst

def f30():
    Baz

def f31():
    f30()

def f32(p):
    if p[1:2] == ':':
        return p[0:2], p[2:]
    return '', p

def f33(n):
    if ((n+1) & n) == 0:
        print n

def f34(m=2000000):
    for i in xrange(m):
        f33(i)

def f35():
    try:
        import Numeric
    except ImportError:
        print "[[5 5 5]"
        print " [5 5 5]"
        print " [5 5 5]]"
    else:
        a = Numeric.zeros((3,3))
        a += 5
        print a

def f36():
    class X:
        pass
    class Y(X):
        pass
    return Y(), Y.__module__, __in_psyco__

def f37(x):
    print 'ZERO' in globals().keys()
    dir()
    print eval('ZERO+1')

def f38(n):
    f = f1
    r = [
        f(n, *[]),
        f(n),
        apply(f, (n,)),
        apply(f, [n]),
        f(*(n,)),
        f(*[n]),
        f(n=n),
        f(**{'n': n}),
        apply(f, (n,), {}),
        apply(f, [n], {}),
        f(*(n,), **{}),
        f(*[n], **{}),
        f(n, **{}),
        f(n, *[], **{}),
        f(n=n, **{}),
        f(n=n, *[], **{}),
        f(*(n,), **{}),
        f(*[n], **{}),
        f(*[], **{'n':n}),
        ]
    return r

if __name__ == "__main__":
    print "Break!"
    sleep(0.5)
    #go(f36)
    #go(f38, 100)
    psyco.full()
    try:
        print f35()
        #f23(0.0)
    finally:
        psyco.dumpcodebuf()



###########################################################################
# 
#  Psyco logger.
#   Copyright (C) 2001-2002  Armin Rigo et.al.

"""Psyco logger.

See log() in core.py.
"""
###########################################################################


import _psyco
from time import time, localtime, strftime


current = None
print_charges = 10
dump_delay = 0.2
dump_last = 0.0

def write(s, level):
    t = time()
    f = t-int(t)
    try:
        current.write("%s.%02d  %-*s%s\n" % (
            strftime("%X", localtime(int(t))),
            int(f*100.0), 63-level, s,
            "%"*level))
        current.flush()
    except (OSError, IOError):
        pass

def psycowrite(s):
    t = time()
    f = t-int(t)
    try:
        current.write("%s.%02d  %-*s%s\n" % (
            strftime("%X", localtime(int(t))),
            int(f*100.0), 60, s.strip(),
            "% %"))
        current.flush()
    except (OSError, IOError):
        pass

##def writelines(lines, level=0):
##    if lines:
##        t = time()
##        f = t-int(t)
##        timedesc = strftime("%x %X", localtime(int(t)))
##        print >> current, "%s.%03d  %-*s %s" % (
##            timedesc, int(f*1000),
##            50-level, lines[0],
##            "+"*level)
##        timedesc = " " * (len(timedesc)+5)
##        for line in lines[1:]:
##            print >> current, timedesc, line

def writememory():
    write("memory usage: %d+ kb" % _psyco.memory(), 1)

def dumpcharges():
    global dump_last
    if print_charges:
        t = time()
        if not (dump_last <= t < dump_last+dump_delay):
            if t <= dump_last+1.5*dump_delay:
                dump_last += dump_delay
            else:
                dump_last = t
            #write("%s: charges:" % who, 0)
            lst = _psyco.stattop(print_charges)
            if lst:
                f = t-int(t)
                lines = ["%s.%02d   ______\n" % (
                    strftime("%X", localtime(int(t))),
                    int(f*100.0))]
                i = 1
                for co, charge in lst:
                    detail = co.co_filename
                    if len(detail) > 19:
                        detail = '...' + detail[-17:]
                    lines.append("        #%-3d |%4.1f %%|  %-26s%20s:%d\n" %
                                 (i, charge*100.0, co.co_name, detail,
                                  co.co_firstlineno))
                    i += 1
                current.writelines(lines)
                current.flush()

def writefinalstats():
    dumpcharges()
    writememory()
    writedate("program exit")

def writedate(msg):
    write('%s, %s' % (msg, strftime("%x")), 20)



#!/usr/bin/python
# Back-Propagation Neural Networks
# 
# Written in Python.  See http://www.python.org/
#
# Neil Schemenauer <nascheme@enme.ucalgary.ca>

import math
import random
import operator
import string
import psyco
#psyco.full()
from psyco.classes import *
#psyco.log()
#psyco.profile()
#__metaclass__ = type


random.seed(0)

def time(fn, *args):
	import time, traceback
	begin = time.clock()
	try:
		result = fn(*args)
	except:
		end = time.clock()
		traceback.print_exc()
		result = '<exception>'
	else:
		end = time.clock()
	return result, end-begin

# calculate a random number where:  a <= rand < b
def rand(a, b):
    return (b-a)*random.random() + a

# Make a matrix (we could use NumPy to speed this up)
def makeMatrix(I, J, fill=0.0):
    m = []
    for i in range(I):
        m.append([fill]*J)
    return m

class NN:
    
    def __init__(self, ni, nh, no):
        # number of input, hidden, and output nodes
        self.ni = ni + 1 # +1 for bias node
        self.nh = nh
        self.no = no

        # activations for nodes
        self.ai = [1.0]*self.ni
        self.ah = [1.0]*self.nh
        self.ao = [1.0]*self.no
        
        # create weights
        self.wi = makeMatrix(self.ni, self.nh)
        self.wo = makeMatrix(self.nh, self.no)
        # set them to random vaules
        for i in range(self.ni):
            for j in range(self.nh):
                self.wi[i][j] = rand(-2.0, 2.0)
        for j in range(self.nh):
            for k in range(self.no):
                self.wo[j][k] = rand(-2.0, 2.0)

        # last change in weights for momentum   
        self.ci = makeMatrix(self.ni, self.nh)
        self.co = makeMatrix(self.nh, self.no)

    def update(self, inputs):
        if len(inputs) != self.ni-1:
            raise ValueError, 'wrong number of inputs'

        # input activations
        for i in range(self.ni-1):
            #self.ai[i] = 1.0/(1.0+math.exp(-inputs[i]))
            self.ai[i] = inputs[i]

        # hidden activations
        for j in range(self.nh):
            sum = 0.0
            for i in range(self.ni):
                sum = sum + self.ai[i] * self.wi[i][j]
            self.ah[j] = 1.0/(1.0+math.exp(-sum))

        # output activations
        for k in range(self.no):
            sum = 0.0
            for j in range(self.nh):
                sum = sum + self.ah[j] * self.wo[j][k]
            self.ao[k] = 1.0/(1.0+math.exp(-sum))

        return self.ao[:]


    def backPropagate(self, targets, N, M):
        if len(targets) != self.no:
            raise ValueError, 'wrong number of target values'

        # calculate error terms for output
        output_deltas = [0.0] * self.no
        for k in range(self.no):
            ao = self.ao[k]
            output_deltas[k] = ao*(1-ao)*(targets[k]-ao)

        # calculate error terms for hidden
        hidden_deltas = [0.0] * self.nh
        for j in range(self.nh):
            sum = 0.0
            for k in range(self.no):
                sum = sum + output_deltas[k]*self.wo[j][k]
            hidden_deltas[j] = self.ah[j]*(1-self.ah[j])*sum

        # update output weights
        for j in range(self.nh):
            for k in range(self.no):
                change = output_deltas[k]*self.ah[j]
                self.wo[j][k] = self.wo[j][k] + N*change + M*self.co[j][k]
                self.co[j][k] = change
                #print N*change, M*self.co[j][k]

        # update input weights
        for i in range(self.ni):
            for j in range(self.nh):
                change = hidden_deltas[j]*self.ai[i]
                self.wi[i][j] = self.wi[i][j] + N*change + M*self.ci[i][j]
                self.ci[i][j] = change

        # calculate error
        error = 0.0
        for k in range(len(targets)):
            error = error + 0.5*(targets[k]-self.ao[k])**2
        return error


    def test(self, patterns):
        for p in patterns:
            print p[0], '->', self.update(p[0])

    def weights(self):
        print 'Input weights:'
        for i in range(self.ni):
            print self.wi[i]
        print
        print 'Output weights:'
        for j in range(self.nh):
            print self.wo[j]

    def train(self, patterns, iterations=2000, N=0.5, M=0.1):
        # N: learning rate
        # M: momentum factor
        for i in xrange(iterations):
            error = 0.0
            for p in patterns:
                inputs = p[0]
                targets = p[1]
                self.update(inputs)
                error = error + self.backPropagate(targets, N, M)
            if i % 100 == 0:
                print 'error %-14f' % error


def demo():
    # Teach network XOR function
    pat = [
        [[0,0], [0]],
        [[0,1], [1]],
        [[1,0], [1]],
        [[1,1], [0]]
    ]

    # create a network with two input, two hidden, and two output nodes
    n = NN(2, 3, 1)
    # train it with some patterns
    n.train(pat, 2000)
    # test it
    n.test(pat)



if __name__ == '__main__':
    v, t1 = time(demo)
    v, t2 = time(demo)
    v, t3 = time(demo)
    v, t4 = time(demo)
    v, t5 = time(demo)
    print t1, t2, t3, t4, t5
    psyco.dumpcodebuf()



import sys, os, test.regrtest

import psyco
import time; print "Break!"; time.sleep(0.5)
psyco.log()
if os.path.exists('regrtester.local'):
    execfile('regrtester.local')
else:
    psyco.full()


#################################################################################
SKIP = {'test_gc': "test_gc.test_frame() does not create a cycle with Psyco's limited frames",
        'test_descr': 'seems that it mutates user-defined types and Psyco does not like it at all',
        'test_profilehooks': 'no profiling allowed with Psyco!',
        'test_profile': 'no profiling allowed with Psyco!',
        'test_cProfile': 'no profiling allowed with Psyco!',
        'test_repr': 'self-nested tuples and lists not supported',
        'test_trace': 'no line tracing with Psyco',
        'test_threaded_import': 'Python hang-ups',
        'test_hotshot': "PyEval_SetProfile(NULL,NULL) doesn't allow Psyco to take control back",
        'test_richcmp': 'uses eval() with locals and circular data structure cmps',
        'test_longexp': 'run it separately if you want, but it takes time and memory',
        'test_weakref': 'only run with FULL_CONTROL_FLOW set to 0 in mergepoints.c',
        'test_gettext': 'gettext mutates _ in the builtins!',
        'test_inspect': 'isframe() does not recognize our Frame instances',
        'test_exceptions': 'uses tb.tb_frame.f_back',
        'test_largefile': 'fails on Python on my old Linux box',
        'test_popen2': 'log file descriptor messed up in Python < 2.2.2',
        'test_sys': 'getrefcount() cannot be reliably tested',
        'test_socket': 'refcounting stuff as well',
        #'test_copy': 'xrange() is very similar to range() with Psyco',
        'test_tarfile': 'we get permission denied with Python',
        'test_scope': 'refcounting: relies on the __del__ of instances',
        'test_sax': 'fails without Psyco due to my Expat installation',
        'test_email': 'broken? on \n vs. \r\n',
        'test_mmap': '"invalid handle" with Python 2.2.2',
        'test_winsound': 'winsound fails on Python 2.3 on my machine',
        'test_strptime': "fails about my timezone ending in (heure d'ete) in 2.3",
        'test_atexit': "windows: tired to work around w9xpopen magic",
        'test_popen': "windows: tired to work around w9xpopen magic",
        'test_mimetypes': 'fail in 2.3 if run after test_urllib',
        'test_doctest': 'doctest uses the debugger!?',
        'test_subprocess': 'suspect subprocess messes up stdout with Psyco messages',
        'test_linuxaudiodev': 'this /dev is absent or unreliable on a few machines of mine',
        'test_tcl': 'requires $DISPLAY',
        'test_pep352': 'no warning for string exceptions',
        'test_ctypes': 'it is full of sys.getrefcount() assertions',
        'test_import': 'I hack my pythons to not import bare .pyc files any more',
        'test_runpy':  'I hack my pythons to not import bare .pyc files any more',
        'test_genexps': 'doctests checking the exact error message',
	}
#    SKIP['test_operator'] = NO_SYS_EXC
#    SKIP['test_strop'] = NO_SYS_EXC
#if sys.version_info[:2] >= (2,3):
#    SKIP['test_threadedtempfile'] = 'Python bug: Python test just hangs up'

if sys.version_info[:3] == (2,4,0):
    SKIP['test_distutils'] = 'distutils/tests/* not copied by the installer'

if hasattr(psyco._psyco, 'VERBOSE_LEVEL'):
    SKIP['test_popen2'] = 'gets confused by Psyco debugging output to stderr'

if os.path.exists('regrtester.skip'):
    execfile('regrtester.skip')

#################################################################################


# the tests that don't work with Psyco
test.regrtest.NOTTESTS += SKIP.keys()
if __name__ == '__main__':
    try:
        test.regrtest.main(randomize=1)
    finally:
        psyco.dumpcodebuf()



import os

dir = os.path.join(os.path.dirname(__file__), 'prolog')
for filename in os.listdir(dir):
    if filename.lower().endswith('.default'):
        basename = os.path.join(dir, filename[:-8])
        filename = os.path.join(dir, filename)
        if not os.path.exists(basename):
            copy = 2
        else:
            try:
                copy = os.stat(filename).st_mtime > os.stat(basename).st_mtime
            except OSError:
                copy = 0
        if copy:
            print 'Copying', filename, '->', basename
            if copy != 2:
                print '(OVERRIDING the older file)'
            import shutil
            shutil.copy(filename, basename)



import re


explicitnumber = {
    '0x010100f0': '1.1.1',   # bugfix release, no change in hex version
    '0x010200f0': '1.2',
    '0x010300f0': '1.3',
    '0x010400f0': '1.4',
    '0x010500f0': '1.5',
    '0x010501f0': '1.5.1',
    '0x010502f0': '1.5.2',
    '0x010600f0': '1.6',
    }

FILES = {
    'Python/pyver.h': re.compile(r'PSYCO_VERSION_HEX\s+(0x[0-9a-fA-F]+)'),
    '../py-support/support.py': re.compile(r'__version__\s*=\s*(0x[0-9a-fA-F]+)'),
    '../doc/psycoguide.tex': re.compile(r'\\release\{([0-9.]+)\}'),
    '../setup.py': re.compile(r'version\s*=\s*\"([0-9.]+)\"'),
    '../../www/dist/Makefile': re.compile(r'version\s*=\s*([0-9.]+)'),
    '../../www/content/index.rst': re.compile(r'`Psyco ([0-9.]+)@'),
    '../../www/content/download.rst': re.compile(r'Current version is ([0-9.]+)'),
    '../../www/content/./download.rst': re.compile(r'[dD]ownload Release ([0-9.]+)'),
    '../../www/content/doc.rst': re.compile(r'Psyco release ([0-9.]+)'),
    '../README.txt': re.compile(r'VERSION ([0-9.]+)'),
    }

versions = {}
for filename, regexp  in FILES.items():
    for line in open(filename, 'r'):
        match = regexp.search(line)
        if match:
            break
    else:
        raise Exception, "No version number found in " + filename
    ver = match.group(1)
    print '%20s  %s' % (ver, filename)
    if ver.startswith('0x'):
        ver = explicitnumber[ver]
    versions[ver] = filename

if len(versions) != 1:
    raise Exception, versions
else:
    print "versionchecker: ok"

import files; files.main()



import sys, os

def getincl():
    PY_VER = sys.version[:3]
    for p in (sys.prefix, sys.exec_prefix):
        if sys.executable.startswith(p):
            return [os.path.join(sys.prefix, 'include', 'python'+PY_VER)]
    base = os.path.dirname(sys.executable)
    if os.path.exists(os.path.join(base, 'Include')):
        return [os.path.join(base, 'Include'), base, os.path.join(base, 'Stackless')]
    raise IOError, 'cannot find the include directories for %s' % sys.executable

INCLUDE_STR = ' '.join(['-I%s' % s for s in getincl()])

if __name__ == '__main__':
    print INCLUDE_STR



###########################################################################
# 
#  Psyco profiler (Python part).
#   Copyright (C) 2001-2002  Armin Rigo et.al.

"""Psyco profiler (Python part).

The implementation of the non-time-critical parts of the profiler.
See profile() and full() in core.py for the easy interface.
"""
###########################################################################

import _psyco
from support import *
import math, time, types, atexit
now = time.time
try:
    import thread
except ImportError:
    import dummy_thread as thread


# current profiler instance
current = None

# enabled profilers, in order of priority
profilers = []

# logger module (when enabled by core.log())
logger = None

# a lock for a thread-safe go()
go_lock = thread.allocate_lock()

def go(stop=0):
    # run the highest-priority profiler in 'profilers'
    global current
    go_lock.acquire()
    try:
        prev = current
        if stop:
            del profilers[:]
        if prev:
            if profilers and profilers[0] is prev:
                return    # best profiler already running
            prev.stop()
            current = None
        for p in profilers[:]:
            if p.start():
                current = p
                if logger: # and p is not prev:
                    logger.write("%s: starting" % p.__class__.__name__, 5)
                return
    finally:
        go_lock.release()
    # no profiler is running now
    if stop:
        if logger:
            logger.writefinalstats()
    else:
        tag2bind()

atexit.register(go, 1)


def buildfncache(globals, cache):
    if hasattr(types.IntType, '__dict__'):
        clstypes = (types.ClassType, types.TypeType)
    else:
        clstypes = types.ClassType
    for x in globals.values():
        if isinstance(x, types.MethodType):
            x = x.im_func
        if isinstance(x, types.FunctionType):
            cache[x.func_code] = x, ''
        elif isinstance(x, clstypes):
            for y in x.__dict__.values():
                if isinstance(y, types.MethodType):
                    y = y.im_func
                if isinstance(y, types.FunctionType):
                    cache[y.func_code] = y, x.__name__

# code-to-function mapping (cache)
function_cache = {}

def trytobind(co, globals, log=1):
    try:
        f, clsname = function_cache[co]
    except KeyError:
        buildfncache(globals, function_cache)
        try:
            f, clsname = function_cache[co]
        except KeyError:
            if logger:
                logger.write('warning: cannot find function %s in %s' %
                             (co.co_name, globals.get('__name__', '?')), 3)
            return  # give up
    if logger and log:
        modulename = globals.get('__name__', '?')
        if clsname:
            modulename += '.' + clsname
        logger.write('bind function: %s.%s' % (modulename, co.co_name), 1)
    f.func_code = _psyco.proxycode(f)


# the list of code objects that have been tagged
tagged_codes = []

def tag(co, globals):
    if logger:
        try:
            f, clsname = function_cache[co]
        except KeyError:
            buildfncache(globals, function_cache)
            try:
                f, clsname = function_cache[co]
            except KeyError:
                clsname = ''  # give up
        modulename = globals.get('__name__', '?')
        if clsname:
            modulename += '.' + clsname
        logger.write('tag function: %s.%s' % (modulename, co.co_name), 1)
    tagged_codes.append((co, globals))
    _psyco.turbo_frame(co)
    _psyco.turbo_code(co)

def tag2bind():
    if tagged_codes:
        if logger:
            logger.write('profiling stopped, binding %d functions' %
                         len(tagged_codes), 2)
        for co, globals in tagged_codes:
            trytobind(co, globals, 0)
        function_cache.clear()
        del tagged_codes[:]


class Profiler:
    MemoryTimerResolution = 0.103

    def run(self, memory, time, memorymax, timemax):
        self.memory = memory
        self.memorymax = memorymax
        self.time = time
        if timemax is None:
            self.endtime = None
        else:
            self.endtime = now() + timemax
        self.alarms = []
        profilers.append(self)
        go()
    
    def start(self):
        curmem = _psyco.memory()
        memlimits = []
        if self.memorymax is not None:
            if curmem >= self.memorymax:
                if logger:
                    logger.writememory()
                return self.limitreached('memorymax')
            memlimits.append(self.memorymax)
        if self.memory is not None:
            if self.memory <= 0:
                if logger:
                    logger.writememory()
                return self.limitreached('memory')
            memlimits.append(curmem + self.memory)
            self.memory_at_start = curmem

        curtime = now()
        timelimits = []
        if self.endtime is not None:
            if curtime >= self.endtime:
                return self.limitreached('timemax')
            timelimits.append(self.endtime - curtime)
        if self.time is not None:
            if self.time <= 0.0:
                return self.limitreached('time')
            timelimits.append(self.time)
            self.time_at_start = curtime
        
        try:
            self.do_start()
        except error, e:
            if logger:
                logger.write('%s: disabled by psyco.error:' % (
                    self.__class__.__name__), 4)
                logger.write('    %s' % str(e), 3)
            return 0
        
        if memlimits:
            self.memlimits_args = (time.sleep, (self.MemoryTimerResolution,),
                                   self.check_memory, (min(memlimits),))
            self.alarms.append(_psyco.alarm(*self.memlimits_args))
        if timelimits:
            self.alarms.append(_psyco.alarm(time.sleep, (min(timelimits),),
                                            self.time_out))
        return 1
    
    def stop(self):
        for alarm in self.alarms:
            alarm.stop(0)
        for alarm in self.alarms:
            alarm.stop(1)   # wait for parallel threads to stop
        del self.alarms[:]
        if self.time is not None:
            self.time -= now() - self.time_at_start
        if self.memory is not None:
            self.memory -= _psyco.memory() - self.memory_at_start

        try:
            self.do_stop()
        except error:
            return 0
        return 1

    def check_memory(self, limit):
        if _psyco.memory() < limit:
            return self.memlimits_args
        go()

    def time_out(self):
        self.time = 0.0
        go()

    def limitreached(self, limitname):
        try:
            profilers.remove(self)
        except ValueError:
            pass
        if logger:
            logger.write('%s: disabled (%s limit reached)' % (
                self.__class__.__name__, limitname), 4)
        return 0


class FullCompiler(Profiler):

    def do_start(self):
        _psyco.profiling('f')

    def do_stop(self):
        _psyco.profiling('.')


class RunOnly(Profiler):

    def do_start(self):
        _psyco.profiling('n')

    def do_stop(self):
        _psyco.profiling('.')


class ChargeProfiler(Profiler):

    def __init__(self, watermark, parentframe):
        self.watermark = watermark
        self.parent2 = parentframe * 2.0
        self.lock = thread.allocate_lock()

    def init_charges(self):
        _psyco.statwrite(watermark = self.watermark,
                         parent2   = self.parent2)

    def do_stop(self):
        _psyco.profiling('.')
        _psyco.statwrite(callback = None)


class ActiveProfiler(ChargeProfiler):

    def active_start(self):
        _psyco.profiling('p')

    def do_start(self):
        self.init_charges()
        self.active_start()
        _psyco.statwrite(callback = self.charge_callback)

    def charge_callback(self, frame, charge):
        tag(frame.f_code, frame.f_globals)


class PassiveProfiler(ChargeProfiler):

    initial_charge_unit   = _psyco.statread('unit')
    reset_stats_after     = 120      # half-lives (maximum 200!)
    reset_limit           = initial_charge_unit * (2.0 ** reset_stats_after)

    def __init__(self, watermark, halflife, pollfreq, parentframe):
        ChargeProfiler.__init__(self, watermark, parentframe)
        self.pollfreq = pollfreq
        # self.progress is slightly more than 1.0, and computed so that
        # do_profile() will double the change_unit every 'halflife' seconds.
        self.progress = 2.0 ** (1.0 / (halflife * pollfreq))

    def reset(self):
        _psyco.statwrite(unit = self.initial_charge_unit, callback = None)
        _psyco.statreset()
        if logger:
            logger.write("%s: resetting stats" % self.__class__.__name__, 1)

    def passive_start(self):
        self.passivealarm_args = (time.sleep, (1.0 / self.pollfreq,),
                                  self.do_profile)
        self.alarms.append(_psyco.alarm(*self.passivealarm_args))

    def do_start(self):
        tag2bind()
        self.init_charges()
        self.passive_start()

    def do_profile(self):
        _psyco.statcollect()
        if logger:
            logger.dumpcharges()
        nunit = _psyco.statread('unit') * self.progress
        if nunit > self.reset_limit:
            self.reset()
        else:
            _psyco.statwrite(unit = nunit, callback = self.charge_callback)
        return self.passivealarm_args

    def charge_callback(self, frame, charge):
        trytobind(frame.f_code, frame.f_globals)


class ActivePassiveProfiler(PassiveProfiler, ActiveProfiler):

    def do_start(self):
        self.init_charges()
        self.active_start()
        self.passive_start()

    def charge_callback(self, frame, charge):
        tag(frame.f_code, frame.f_globals)



#
# we register our own version of sys.settrace(), sys.setprofile()
# and thread.start_new_thread().
#

def psyco_settrace(*args, **kw):
    "This is the Psyco-aware version of sys.settrace()."
    result = original_settrace(*args, **kw)
    go()
    return result

def psyco_setprofile(*args, **kw):
    "This is the Psyco-aware version of sys.setprofile()."
    result = original_setprofile(*args, **kw)
    go()
    return result

def psyco_thread_stub(callable, args, kw):
    _psyco.statcollect()
    if kw is None:
        return callable(*args)
    else:
        return callable(*args, **kw)

def psyco_start_new_thread(callable, args, kw=None):
    "This is the Psyco-aware version of thread.start_new_thread()."
    return original_start_new_thread(psyco_thread_stub, (callable, args, kw))

original_settrace         = sys.settrace
original_setprofile       = sys.setprofile
original_start_new_thread = thread.start_new_thread
sys.settrace            = psyco_settrace
sys.setprofile          = psyco_setprofile
thread.start_new_thread = psyco_start_new_thread
# hack to patch threading._start_new_thread if the module is
# already loaded
if (sys.modules.has_key('threading') and
    hasattr(sys.modules['threading'], '_start_new_thread')):
    sys.modules['threading']._start_new_thread = psyco_start_new_thread



#!/usr/bin/env python
# life.py -- A curses-based version of Conway's Game of Life.
# Contributed by AMK
#
# An empty board will be displayed, and the following commands are available:
#  E : Erase the board
#  R : Fill the board randomly
#  S : Step for a single generation
#  C : Update continuously until a key is struck
#  Q : Quit
#  Cursor keys :  Move the cursor around the board
#  Space or Enter : Toggle the contents of the cursor's position
#
# TODO : 
#   Support the mouse
#   Use colour if available
#   Make board updates faster
#

import psyco; psyco.full()


class LifeBoard:
    """Encapsulates a Life board

    Attributes:
    X,Y : horizontal and vertical size of the board
    state : dictionary mapping (x,y) to 0 or 1
    
    Methods:
    display(update_board) -- If update_board is true, compute the 
                             next generation.  Then display the state
			     of the board and refresh the screen.
    erase() -- clear the entire board
    makeRandom() -- fill the board randomly
    set(y,x) -- set the given cell to Live; doesn't refresh the screen
    toggle(y,x) -- change the given cell from live to dead, or vice
                   versa, and refresh the screen display

    """
    def __init__(self, scr, char=ord('*')):
	"""Create a new LifeBoard instance.

	scr -- curses screen object to use for display
	char -- character used to render live cells (default: '*')
	"""
	self.state={} ; self.scr=scr
	Y, X = self.scr.getmaxyx()
	self.X, self.Y = X-2, Y-2-1
	self.char = char
	self.scr.clear()	

	# Draw a border around the board
	border_line='+'+(self.X*'-')+'+'
	self.scr.addstr(0, 0, border_line)
	self.scr.addstr(self.Y+1,0, border_line)
	for y in range(0, self.Y): 
	    self.scr.addstr(1+y, 0, '|') 
	    self.scr.addstr(1+y, self.X+1, '|')
	self.scr.refresh()

    def set(self, y, x): 
	"""Set a cell to the live state"""
	if x<0 or self.X<=x or y<0 or self.Y<=y:
	    raise ValueError, "Coordinates out of range %i,%i"% (y,x)
	self.state[x,y] = 1

    def toggle(self, y, x): 
	"""Toggle a cell's state between live and dead"""
	if x<0 or self.X<=x or y<0 or self.Y<=y:
	    raise ValueError, "Coordinates out of range %i,%i"% (y,x)
	if self.state.has_key( (x,y) ): 
	    del self.state[x,y]
	    self.scr.addch(y+1, x+1, ' ')
	else:
	    self.state[x,y]=1
	    self.scr.addch(y+1, x+1, self.char)
	self.scr.refresh()

    def erase(self):
	"""Clear the entire board and update the board display"""
	self.state={}
	self.display(update_board=0)

    def display(self, update_board=1):
	"""Display the whole board, optionally computing one generation"""
	M,N = self.X, self.Y 
	if not update_board:
	    for i in range(0, M):
		for j in range(0, N):
			if self.state.has_key( (i,j) ): 
			    self.scr.addch(j+1, i+1, self.char)
			else:
			    self.scr.addch(j+1, i+1, ' ')
	    self.scr.refresh()
	    return

	d={} ; self.boring=1
	for i in range(0, M):
	    L=range( max(0, i-1), min(M, i+2) )
	    for j in range(0, N):
		s=0
		live=self.state.has_key( (i,j) )
		for k in range( max(0, j-1), min(N, j+2) ):
		    for l in L:
			if self.state.has_key( (l,k) ):
			    s=s+1
		s=s-live
		if s==3: 
		    # Birth
		    d[i,j]=1 
		    self.scr.addch(j+1, i+1, self.char)
		    if not live: self.boring=0  
		elif s==2 and live: d[i,j]=1       # Survival
		elif live: 
		    # Death
		    self.scr.addch(j+1, i+1, ' ')
		    self.boring=0
	self.state=d
	self.scr.refresh()

    def makeRandom(self):
	"Fill the board with a random pattern"
	import whrandom
	self.state={}
	for i in range(0, self.X): 
            for j in range(0, self.Y):
		if whrandom.random()*10>5.0: self.set(j,i)


def erase_menu(stdscr, menu_y):
    "Clear the space where the menu resides"
    stdscr.move(menu_y, 0) ; stdscr.clrtoeol()
    stdscr.move(menu_y+1, 0) ; stdscr.clrtoeol()

def display_menu(stdscr, menu_y):
    "Display the menu of possible keystroke commands"
    erase_menu(stdscr, menu_y)
    stdscr.addstr(menu_y, 4,
                  'Use the cursor keys to move, and space or Enter to toggle a cell.')
    stdscr.addstr(menu_y+1, 4,
                  'E)rase the board, R)andom fill, S)tep once or C)ontinuously, Q)uit')

def main(stdscr):
    import string, curses

    # Clear the screen and display the menu of keys
    stdscr.clear()
    stdscr_y, stdscr_x = stdscr.getmaxyx()
    menu_y=(stdscr_y-3)-1
    display_menu(stdscr, menu_y)

    # Allocate a subwindow for the Life board and create the board object
    subwin=stdscr.subwin(stdscr_y-3, stdscr_x, 0, 0) 
    board=LifeBoard(subwin, char=ord('*'))
    board.display(update_board=0)

    # xpos, ypos are the cursor's position
    xpos, ypos = board.X/2, board.Y/2

    # Main loop:
    while (1):
	stdscr.move(1+ypos, 1+xpos)     # Move the cursor
	c=stdscr.getch()		# Get a keystroke
	if 0<c<256:
	    c=chr(c)
	    if c in ' \n':
		board.toggle(ypos, xpos)
	    elif c in 'Cc':
		erase_menu(stdscr, menu_y)
		stdscr.addstr(menu_y, 6, ' Hit any key to stop continuously '
			      'updating the screen.')
		stdscr.refresh()
		# Activate nodelay mode; getch() will return -1
		# if no keystroke is available, instead of waiting.
		stdscr.nodelay(1)   
		while (1):
		    c=stdscr.getch()
		    if c!=-1: break
		    stdscr.addstr(0,0, '/'); stdscr.refresh()
		    board.display()
		    stdscr.addstr(0,0, '+'); stdscr.refresh()

		stdscr.nodelay(0)	# Disable nodelay mode
		display_menu(stdscr, menu_y)

	    elif c in 'Ee': board.erase()
	    elif c in 'Qq': break
	    elif c in 'Rr': 
		board.makeRandom()
		board.display(update_board=0)
	    elif c in 'Ss':
		board.display()
	    else: pass                  # Ignore incorrect keys
	elif c==curses.KEY_UP and ypos>0: 	     ypos=ypos-1
	elif c==curses.KEY_DOWN and ypos<board.Y-1:  ypos=ypos+1
	elif c==curses.KEY_LEFT and xpos>0:          xpos=xpos-1
	elif c==curses.KEY_RIGHT and xpos<board.X-1: xpos=xpos+1
        else: pass			# Ignore incorrect keys

if __name__=='__main__':
    import curses, traceback
    try:
	# Initialize curses
	stdscr=curses.initscr()
	# Turn off echoing of keys, and enter cbreak mode,
	# where no buffering is performed on keyboard input
	curses.noecho() ; curses.cbreak()

	# In keypad mode, escape sequences for special keys
	# (like the cursor keys) will be interpreted and
	# a special value like curses.KEY_LEFT will be returned
	stdscr.keypad(1)
	main(stdscr)			# Enter the main loop
	# Set everything back to normal
	stdscr.keypad(0)
	curses.echo() ; curses.nocbreak()
	curses.endwin()			# Terminate curses
    except:
        # In the event of an error, restore the terminal
	# to a sane state.
	stdscr.keypad(0)
	curses.echo() ; curses.nocbreak()
	curses.endwin()
	traceback.print_exc()		# Print the exception




#! /usr/bin/env python

"""  Run base Psyco tests.
"""

import sys, os, random, doctest, cStringIO
True,False = 1==1,1==0


SEPARATOR = """
========== %r ==========
"""
LASTLINE = "Tests completed."
BUFFERFILE = "buffer-basetests.txt"
EXPECTEDFILE = "expected-basetests.txt"
INPUTSCRIPT = "input-basetexts.py"

TESTS = open('btrun.py', 'r').read()


if hasattr(doctest, '_extract_examples'):
    tests = doctest._extract_examples(TESTS)
else:
    examples = doctest.DocTestParser().get_examples(TESTS)
    tests = [(example.source, example.want, example.lineno)
             for example in examples]

# ---

def main(quiet=False):
    PRELUDE = ''
    for inp, outp, line in tests[:]:
        if not outp:
            PRELUDE += inp + '\n'
            tests.remove((inp, outp, line))
    random.shuffle(tests)             # first run all tests in any order
    tests_again = tests[:]
    random.shuffle(tests_again)
    all_tests = tests + tests_again   # then run them all again in any other order

    childin = open(INPUTSCRIPT, 'w')
    expected = open(EXPECTEDFILE, 'w')

    print >> childin, 'import sys'
    print >> childin, PRELUDE

    def filterline(line):
        if line.startswith('${') and line.endswith('}'):
            line = str(eval(line[2:-1]))
        return line

    for inp, outp, line in all_tests:
        sep = SEPARATOR % inp
        print >> childin, 'print %r' % sep
        if not quiet:
            print >> childin, 'print >> sys.stderr, %r' % inp.strip()
        print >> expected, sep
        print >> childin, inp
        outplines = [filterline(line) for line in outp.split('\n')]
        expected.write('\n'.join(outplines))

    print >> childin, 'print %r' % LASTLINE
    print >> expected, LASTLINE
    expected.close()
    childin.close()

    # run in a child process
    err = os.system('"%s" %s > %s' % (sys.executable, INPUTSCRIPT, BUFFERFILE))
    print >> sys.stderr
    if err:
        print >> sys.stderr, 'FAIL: child process returned %d, %d' % (err>>8, err&255)
        sys.exit(1)
    else:
        data1 = open(EXPECTEDFILE, 'r').read()
        data2 = open(BUFFERFILE, 'r').read()
        if data1 != data2:
            print >> sys.stderr, 'FAIL: different output'
            if sys.argv[1:2] != ['-q']:
                cmd = 'diff -c %s %s' % (EXPECTEDFILE, BUFFERFILE)
                os.system(cmd)
            return False
        else:
            print >> sys.stderr, 'Passed.'
            return True

def test_main():
    ok = main(quiet=True)
    assert ok

if __name__ == '__main__':
    if not main():
        sys.exit(1)



import py, sys, psyco


def test_index():
    if sys.version_info < (2, 5):
        py.test.skip("for Python 2.5")

    class X(object):
        def __index__(self):
            return -3

    def f(x):
        return (12, 23, 34, 45)[x]

    res = psyco.proxy(f)(X())
    assert res == 23


def test_index_slice():
    if sys.version_info < (2, 5):
        py.test.skip("for Python 2.5")

    class Observer(object):
        def __getslice__(self, i, j):
            return "slice", i, j
        def __getitem__(self, x):
            raise Exception("in __getitem__")
    class X(object):
        def __index__(self):
            return 5
    class Y(object):
        def __index__(self):
            return -2

    def f(o, x, y):
        return o[x:y]

    res = psyco.proxy(f)(Observer(), X(), Y())
    assert res == ("slice", 5, -2)

def test_index_repeat():
    if sys.version_info < (2, 5):
        py.test.skip("for Python 2.5")

    class X(object):
        def __index__(self):
            return 3

    def f(x):
        return (12, 23) * x

    res = psyco.proxy(f)(X())
    assert res == (12, 23, 12, 23, 12, 23)

def test_slice_overflow():
    class X(object):
        def __len__(self):
            return sys.maxint
        def __getslice__(self, i, j):
            return i, j
    def f(X):
        return X()[-(2**100):2**100]

    res = psyco.proxy(f)(X)
    assert res == f(X)

def test_id():
    def f(x):
        return id(x)
    obj = []
    res = psyco.proxy(f)(obj)
    assert res == id(obj)

def test_math_module():
    import math
    global _operation
    for name in ["acos", "asin", "atan", "ceil", "cos", "cosh", "exp",
                 "fabs", "floor", "sin", "sinh", "sqrt", "tan", "tanh"]:
        _operation = getattr(math, name)

        def tester(a):
            return _operation(a)
        res = psyco.proxy(tester)(0.17)
        assert abs(res - _operation(0.17)) < 1e6

        def tester(a):
            return _operation(*a)
        res = psyco.proxy(tester)((0.71,))
        assert abs(res - _operation(0.71)) < 1e6

    for name in ["atan2", "fmod", "hypot", "pow"]:
        _operation = getattr(math, name)

        def tester(a, b):
            return _operation(a, b)
        res = psyco.proxy(tester)(0.17, 0.081)
        assert abs(res - _operation(0.17, 0.081)) < 1e6

        def tester(a):
            return _operation(*a)
        res = psyco.proxy(tester)((0.71, 0.2643))
        assert abs(res - _operation(0.71, 0.2643)) < 1e6



#! /usr/bin/env python

"""Psyco shows that it is possible to execute Python code at speeds
approaching that of fully compiled languages, by "specialization".
This extension module for the unmodified interpreter accelerates
user programs with little or not change in their sources, by a
factor that can be very interesting (2-10 times is common)."""

import os, sys
from distutils.core import setup
from distutils.extension import Extension

PROCESSOR = None  # autodetect

####################################################################
#
#  Customizable debugging flags.
#  Copy the following section in a new file 'preferences.py' to
#  avoiding changing setup.py. Uncomment and change the options there
#  at your will. Setting PsycoDebug to 1 is probably the first thing
#  you want to do to enable all internal checks.

#PSYCO_DEBUG = 1

# level of debugging outputs: 0 = none, 1 = a few, 2 = more,
#   3 = detailled, 4 = full execution trace
#VERBOSE_LEVEL = 0

# write produced blocks of code into a file; see 'xam.py'
#  0 = off, 1 = only manually (from a debugger or with _psyco.dumpcodebuf()),
#  2 = only when returning from Psyco,
#  3 = every time a new code block is built
#CODE_DUMP = 1

# Linux-only *heavy* memory checking: 0 = off, 1 = reasonably heavy,
#                                     2 = unreasonably heavy.
#HEAVY_MEM_CHECK = 0

# If the following is set to 1, Psyco is compiled by #including all .c
# files into psyco.c.
# It provides a version of _psyco.so whose only exported (non-static)
# symbol is init_psyco(). It also seems that the GDB debugger doesn't locate
# too well non-static symbols in shared libraries. Recompiling after minor
# changes is faster if ALL_STATIC=0.
ALL_STATIC = 1

# Be careful with ALL_STATIC=0, because I am not sure the distutils can
# correctly detect all the dependencies. In case of doubt always compile
# with `setup.py build_ext -f'.


####################################################################

# override options with the ones from preferences.py, if the file exists.
try:
    execfile('preferences.py')
except IOError:
    pass


# processor auto-detection
class ProcessorAutodetectError(Exception):
    pass
def autodetect():
    platform = sys.platform.lower()
    if platform.startswith('win'):   # assume an Intel Windows
        return 'i386'
    # assume we have 'uname'
    mach = os.popen('uname -m', 'r').read().strip()
    if not mach:
        raise ProcessorAutodetectError, "cannot run 'uname -m'"
    if mach == 'x86_64' and sys.maxint == 2147483647:
        mach = 'x86'     # it's a 64-bit processor but in 32-bits mode, maybe
    try:
        return {'i386': 'i386',
                'i486': 'i386',
                'i586': 'i386',
                'i686': 'i386',
                'i86pc': 'i386',    # Solaris/Intel
                'x86':   'i386',    # Apple
                }[mach]
    except KeyError:
        raise ProcessorAutodetectError, "unsupported processor '%s'" % mach


# loads the list of source files from SOURCEDIR/files.py
# and make the appropriate options for the Extension class.
SOURCEDIR = 'c'

data = {}
execfile(os.path.join(SOURCEDIR, 'files.py'), data)

SRC = data['SRC']
MAINFILE = data['MAINFILE']
PLATFILE = data['PLATFILE']

macros = []
for name in ['PSYCO_DEBUG', 'VERBOSE_LEVEL',
             'CODE_DUMP', 'HEAVY_MEM_CHECK', 'ALL_STATIC',
             'PSYCO_NO_LINKED_LISTS']:
    if globals().has_key(name):
        macros.append((name, str(globals()[name])))

if PROCESSOR is None:
    try:
        PROCESSOR = autodetect()
    except ProcessorAutodetectError:
        PROCESSOR = 'ivm'  # fall back to the generic virtual machine
    print "PROCESSOR = %r" % PROCESSOR
processor_dir = os.path.join('c', PROCESSOR)
localsetup = os.path.join(processor_dir, 'localsetup.py')
if os.path.isfile(localsetup):
    d = globals().copy()
    d['__file__'] = localsetup
    execfile(localsetup, d)

if ALL_STATIC:
    sources = [SOURCEDIR + '/' + MAINFILE,
               SOURCEDIR + '/' + PLATFILE]
else:
    sources = [SOURCEDIR + '/' + s.filename for s in SRC]

extra_compile_args = []
extra_link_args = []
if sys.platform == 'win32':
    if globals().get('PSYCO_DEBUG'):
        # how do we know if distutils will use the MS compilers ???
        # these are just hacks that I really need to compile psyco debug versions
        # on Windows
        extra_compile_args.append('/Od')   # no optimizations, override the default /Ox
        extra_compile_args.append('/ZI')   # debugging info
        extra_link_args.append('/debug')   # debugging info
    macros.insert(0, ('NDEBUG', '1'))  # prevents from being linked against python2x_d.lib


CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: C',
    'Topic :: Software Development :: Compilers',
    'Topic :: Software Development :: Interpreters',
    ]

try:
    import distutils.command.register
except ImportError:
    kwds = {}
else:
    kwds = {'classifiers': CLASSIFIERS}

py_modules = ['psyco.%s' % os.path.splitext(fn)[0]
              for fn in os.listdir('py-support')
              if os.path.splitext(fn)[1].lower() == '.py']
py_modules.remove('psyco.__init__')

if sys.version_info < (2, 2, 2):
    raise Exception("Psyco >= 1.6 requires Python >= 2.2.2")


setup (	name             = "psyco",
      	version          = "1.6",
      	description      = "Psyco, the Python specializing compiler",
      	author           = "Armin Rigo",
        author_email     = "arigo@users.sourceforge.net",
      	url              = "http://psyco.sourceforge.net/",
        license          = "MIT License",
        long_description = __doc__,
        platforms        = ["i386"],
        py_modules       = py_modules,
        package_dir      = {'psyco': 'py-support'},
      	ext_modules=[Extension(name = 'psyco._psyco',
                               sources = sources,
                               extra_compile_args = extra_compile_args,
                               extra_link_args = extra_link_args,
                               define_macros = macros,
                               include_dirs = [processor_dir])],
        **kwds )
